/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
 package org.intuitel.merger;

import org.intuitel.imsld.NamespaceMapperImsLd;
import org.intuitel.imsld.lvlA.*;
import org.intuitel.imsld.manifest.Manifest;
import org.intuitel.imsld.manifest.Metadata;
import org.intuitel.imsld.manifest.Organisations;
import org.intuitel.imsld.manifest.Resource;
import org.intuitel.slom.ConceptContainer;
import org.intuitel.slom.KnowledgeObject;
import org.intuitel.slom.MicroLP;
import org.intuitel.slom.SLOM;
import org.intuitel.slom.SLOMUtil;
import org.semanticweb.HermiT.Reasoner;
import org.semanticweb.owlapi.model.*;
import org.semanticweb.owlapi.reasoner.NodeSet;
import org.semanticweb.owlapi.reasoner.OWLReasoner;
import org.semanticweb.owlapi.reasoner.OWLReasonerFactory;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Set;


/**
 * Implements IMSLD serialization.
 *  
 *
 * <p>Further information of the transformation details can be found at the 
 * <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.2</p>
 * 
 * @see Serializer
 * @see IMSLDSerializer 
 * @see SerializerFactory
 */
public class IMSLDSerializer extends XSLTSerializer {
    public static final String PROP_SCHEMA = "schema";
    public static final String VALUE_SCHEMA = "IMS Metadata";
    public static final String PROP_SCHEMA_VERSION = "schemaversion";
    public static final String VALUE_SCHEMA_VERSION = "1.2";

    public static final String VALUE_ROLE_LEARNER = "Learner";
    public static final String VALUE_IMS_LVL = "A";
    public static final String VALUE_IMS_URI = "http://intuitel.eu/public/generatedIMSLD";

    public static final String PREFIX_KO = "Activity_";
    public static final String PREFIX_CC = "AS_";
    public static final String PREFIX_ACT = "ACT_";

    public static final String OWL_KO_TITLE = "title";
    public static final String OWL_KO_LINKED_LMS = "isLinkedWithLmsElement";

    public static final String VALUE_STRUCTURE_TYPE = "sequence";
    
       /**
    * Returns the path to the XSLT file that generates the CCM.
    * 
    */
    protected String getCCMXSLT() {
        return SLOMUtil.IMSLD_TO_CCM_XSLT;
    }
    
   /**
    * Returns the path to the XSLT file that generates the CM.
    * 
    */    
    protected String getCMXSLT() {
        return SLOMUtil.IMSLD_TO_CM_XSLT;
    }
    
   /**
    * Returns the path to the XSLT file that generates the IMS LD manifest file.
    * 
    */    
    protected String getToExternalFormatXSLT() {
        return SLOMUtil.CM_CCM_TO_IMSLD_XSLT;
    }
    
    /**
    * Returns "IMSLD".
    * 
    */    
    protected String getExternalFormatName() {
        return SLOMUtil.IMSLD_NAME;
    }

    
    
    /** 
    * Writes the information stored in a SLOM object in a OutputStream.
    *
    * <p>The marshall method serializes a SLOM object and writes 
    * it as IMS LD manifest package.</p>
    *
    * <p>The method accepts the SLOM object to be serialized, and a Iterator of 
    * OutputStream objects. In the case of IMSLD, only one OutputStream is 
    * required since all the information fits in the imsmanifest.xml file.</p>
    *   
    * <p>The method returns nothing, since the generated information is written in
    * the OutputStream objects passed as parameter.</p>
    *
    * 
    * @param theSlom A SLOM object containing the information to be serialized
    * @param it A Iterator of OutputStream objects, that will be used to write the serialization
    */   
    public void marshall(SLOM theSlom, Iterator<OutputStream> it) throws InvalidParameterException, IOException {
        OutputStream output;
        if (it.hasNext()) {
            output = it.next();
        } else {
            throw new InvalidParameterException("For IMSLD marshalling, at least one OutputStream must be provided");
        }

        String courseId = "generated course";
        String courseTitle = "generated course";

        Manifest imsLdManifest = new Manifest();
        imsLdManifest.setIdentifier(courseId);

        Metadata metadata = new Metadata();
        metadata.setProperty(PROP_SCHEMA, VALUE_SCHEMA);
        metadata.setProperty(PROP_SCHEMA_VERSION, VALUE_SCHEMA_VERSION);
        imsLdManifest.setMetadata(metadata);

        Organisations organisations = new Organisations();
        LearningDesign learningDesign = new LearningDesign();
        learningDesign.setIdentifier(courseId);
        learningDesign.setTitle(courseTitle);
        learningDesign.setLevel(VALUE_IMS_LVL);
        learningDesign.setUri(VALUE_IMS_URI);

        Components components = new Components();
        learningDesign.setComponents(components);

        Activities activities = new Activities();

        components.setRoles(Roles.createLearnerRole(VALUE_ROLE_LEARNER, VALUE_ROLE_LEARNER));
        components.setActivities(activities);

        organisations.setLearningDesign(learningDesign);
        imsLdManifest.setOrganisations(organisations);

        RoleRef roleReference = new RoleRef(VALUE_ROLE_LEARNER);
        
        convertKOs(theSlom, activities.getStructure(), imsLdManifest.getResources());    
        convertCCs(theSlom, activities.getStructure());
        
        Method method = new Method();
        Play play = new Play();
        method.setPlay(play);
        learningDesign.setMethod(method);

        convertPlays(theSlom, play.getAct(), roleReference);

        writeOutput(imsLdManifest, output);
    }

    private void convertKOs(SLOM slom, List<Activity> activities, List<Resource> resources) {
        OWLReasonerFactory reasonerFactory = new Reasoner.ReasonerFactory();
        OWLDataFactory fac = slom.getCCM().getOWLOntologyManager().getOWLDataFactory();
        OWLReasoner reasoner = reasonerFactory.createReasoner(slom.getCCM());

        OWLClass myKOclass = fac.getOWLClass(IRI.create("http://www.intuitel.eu/public/intui_PO.owl#KnowledgeObject"));
        NodeSet<OWLNamedIndividual> individualsKONodeSet = reasoner.getInstances(myKOclass, true);
        Set<OWLNamedIndividual> niKOs = individualsKONodeSet.getFlattened();
        for (OWLNamedIndividual item : niKOs) {
            String resourceIdentifier = item.getIRI().getFragment();
            String activityIdentifier = PREFIX_KO + resourceIdentifier;
            LearningActivity learningActivity = LearningActivity.create(activityIdentifier,
                                                                        getProperty(OWL_KO_TITLE, item, slom.getCCM(),
                                                                                    activityIdentifier),
                                                                        resourceIdentifier);
            activities.add(learningActivity);
            resources.add(Resource.create(resourceIdentifier, getProperty(OWL_KO_LINKED_LMS, item,
                                                                          slom.getCCM(), resourceIdentifier)));
        }
    }

    private void convertCCs(SLOM slom, List<Activity> activities) {
        for (ConceptContainer item : slom.getConceptContainers()) {
            String activityIdentifier = PREFIX_CC + trim(item.getId());
            ActivityStructure activityStructure = new ActivityStructure();
            activityStructure.setIdentifier(activityIdentifier);
            activityStructure.setStructureType(VALUE_STRUCTURE_TYPE);           
            for (MicroLP microLP : slom.getMicroLPs(item)) {            
                for (KnowledgeObject ko : microLP.getKnowledgeObjects()) {
                    LearningActivityRef lar = new LearningActivityRef();
                    String koIdentifier = PREFIX_KO + trim(ko.getId());
                    lar.setRef(koIdentifier);
                    activityStructure.getLearningActivityRef().add(lar);
                }
            }
            activities.add(activityStructure);
        }
    }

    private void convertPlays(SLOM slom, List<Act> acts, RoleRef roleReference) {
        for (ConceptContainer item : slom.getConceptContainers()) {
            String activityIdentifier = PREFIX_CC + trim(item.getId());
            String actIdentifier = PREFIX_ACT + activityIdentifier;
            Act act = new Act();
            act.setIdentifier(actIdentifier);
            RolePart rolePart = new RolePart();
            rolePart.setRoleRef(roleReference);

            ActivityStructureRef activityStructureRef = new ActivityStructureRef();
            activityStructureRef.setRef(activityIdentifier);
            rolePart.setActivityStructureRef(activityStructureRef);

            act.setRolePart(rolePart);

            acts.add(act);
        }
    }

    private String getProperty(String property, OWLNamedIndividual item, OWLOntology ontology, String defaultValue) {
        for (OWLAnnotation annotation : item.getAnnotations(ontology)) {
            if (annotation.getProperty().getIRI().getFragment().equalsIgnoreCase(property)) {
                if (annotation.getValue() instanceof OWLLiteral) {
                    return ((OWLLiteral) annotation.getValue()).getLiteral();
                } else {
                    return trim(annotation.getValue().toString());
                }
            }
        }
        return defaultValue;
    }

    private String trim(String item) {
        if (item == null) {
            return null;
        } else {
            if (item.contains(">")) {
                item = item.replaceAll(">", "");
            }
            return item.substring(item.indexOf('#') + 1);
        }
    }

    private void writeOutput(Manifest imsldManifest, OutputStream output) throws IOException {
        try {
            JAXBContext jc = JAXBContext.newInstance(Manifest.class);
            Marshaller marshaller = jc.createMarshaller();
            marshaller.setProperty(Marshaller.JAXB_SCHEMA_LOCATION, "http://www.imsglobal.org/xsd/imscp_v1p1 " +
                                                                    "http://www.imsglobal.org/xsd/imscp_v1p1p3.xsd " +
                                                                    "http://www.imsglobal.org/xsd/imsld_v1p0 " +
                                                                    "http://www.imsglobal.org/xsd/IMS_LD_Level_A.xsd");
            marshaller.setProperty("com.sun.xml.bind.namespacePrefixMapper", NamespaceMapperImsLd.instance);
            marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
            marshaller.marshal(imsldManifest, output);
        } catch (JAXBException e) {
            throw new IOException(e);
        }
    }

}

