/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */


package org.intuitel.merger;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.Console;
import java.util.Scanner;
import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.HashMap;
import java.util.zip.ZipOutputStream;
import java.util.zip.ZipEntry;
import java.util.LinkedList;
import java.util.Iterator;
import java.net.MalformedURLException;


import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

import org.intuitel.slom.*;

/**
 * Command Line Interface to execute the transformations.
 *
 * 
 * <p>The INTUITEL Merger is a software library that translates course 
    information from and to several formats, with the main goal of 
    obtaining SLOM compliant courses. The MergerCLI class is a command
    line trigger of the funcionality provided by the library</p>

    <p>Therefore, the interface ofered by the MergerCLI allows for the 
    transformations from and to different formats for learning courses. 
    MergerCLI assumes that the courses are stored in a local file, and
    also writes the output in a local file.</p>
 *
 *  <p>To complete a transformation, the user must provide the location 
    (the path) of the input and output files, and also the input and output 
    formats. The tool usage instructions are:</p>

<pre>
USAGE: mergerCLI -f &lt;formatName&gt; -i &lt;path&gt; -o &lt;path&gt; [-p &lt;formatName&gt;]
--------
    -f,--inputFormat &lt;formatName&gt;    The format of the Input file to be transformed. 
                                     Valid options are: 
                                          slom, scorm, smw, imsld

    -i,--inputFile &lt;path&gt;            Input file to be transformed.

    -o,--outputFile &lt;path&gt;           Output file to write the results.
                                     merger_output is the default value

    -p,--outputFormat &lt;formatName&gt;   The format of the Output file. 
                                     Valid options are: 
                                         slom, scorm, imsld

</pre>

 */
public class MergerCLI 
{

   /** 
    * Executes the command line interface.
    *
    * <p>This method triggers the command line interface..</p>
    * 
    * @param args As explained in the class description
    *
    */
    public static void main( String[] args )
    {
    	PrintWriter outputConsole = new PrintWriter(System.out);
    	processCmdLine(args, outputConsole);
    	outputConsole.flush();
    }


    /** 
    * Does the actual processing.
    *
    * <p>This method encapsulates the arguments parsing and processing, doing
    * the actual transformation tasks.</p>
    *
    * <p>As a public method, can be used by other developers who want to write 
    * code that simulate the command line interface usage. This is useful, for example, 
    * to implement JUnit testing procedures.</p>
    * 
    * @param args As explained in the class description
    * @param outputConsole The PrintWriter that will receive the messages to the user (System.out in normal usage)
    *
    */
	public static void processCmdLine(String[] args, PrintWriter outputConsole) {
		Options opts = new Options();
    	
    	@SuppressWarnings("static-access")
		Option inputFile = OptionBuilder.withLongOpt("inputFile")
    									.withArgName("path")
    									.hasArg()
    									.withType(String.class)
    									.hasArg()
    									.withDescription("Input file to be transformed.")
    									.isRequired()
    									.create('i');
    	@SuppressWarnings("static-access")
		Option inputFormat = OptionBuilder.withLongOpt("inputFormat")
    									.withArgName("formatName")
    									.hasArg()
    									.withType(String.class)
    									.hasArg()
    									.withDescription("The format of the Input file to be transformed. Valid options are: slom, scorm, smw, imsld")
    									.isRequired()
    									.create('f');
        @SuppressWarnings("static-access")
		Option outputFile = OptionBuilder.withLongOpt("outputFile")
    									.withArgName("path")
									 	.withType(String.class)
										.withDescription("Output file to write the results. merger_output is the default value")
										.hasArg()
    									.isRequired()
										.create('o');
    	@SuppressWarnings("static-access")
		Option outputFormat = OptionBuilder.withLongOpt("outputFormat")
    									.withArgName("formatName")
    									.hasArg()
    									.withType(String.class)
    									.hasArg()
    									.withDescription("The format of the Output file. Valid options are: slom, scorm, smw, imsld. slom is the default value")
    									.create('p');

    	opts.addOption(inputFile);
    	opts.addOption(inputFormat);
    	opts.addOption(outputFile);
    	opts.addOption(outputFormat);

    	try {
	    CommandLineParser parser = new GnuParser();
	    CommandLine cmd = parser.parse( opts, args);
            String pathIn = cmd.getOptionValue('i');
            String pathOut = cmd.getOptionValue('o');
            String fmtIn = cmd.getOptionValue('f');
            String fmtOut = cmd.getOptionValue('p',"slom");

            Serializer sIn = SerializerFactory.getSerializer(fmtIn);
            Serializer sOut = SerializerFactory.getSerializer(fmtOut);
                        
            File fIn = null;
            if ( fmtIn.equalsIgnoreCase("smw") ) {
                Map<String,String> data = smwGetInput();
                SMWScrapper scrapper = null;
                try {
		   scrapper = new SMWScrapper(pathIn, data.get("user"), data.get("pass"));
		 } catch (MalformedURLException e) {
		  System.err.println("The provided URL is not valid.");
		  e.printStackTrace();
		  System.exit(1);
		 }
		 
                String smwPathIn = scrapper.doScrap();
                fIn = new File(smwPathIn);
            } else {
                //get the File from the path.
                fIn = new File(pathIn);
            }

            //Build the Set<File> 
            Set<File> sfIn = new HashSet<File>();
            sfIn.add(fIn);

            //call the transformation and create the output zip file
            SLOM theSlom = sIn.unmarshall(sfIn);

 
            // the marshalling is case dependand: 
            if ( fmtOut.equalsIgnoreCase("slom") ) {
                slomMarshall(theSlom, sOut, pathOut);
            } else if ( fmtOut.equalsIgnoreCase("imsld") ) {
                imsldMarshall(theSlom, sOut, pathOut);
            } else if ( fmtOut.equalsIgnoreCase("scorm") ) {
                scormMarshall(theSlom, sOut, pathOut);
            } else {
                System.err.println("Should not reach this line, Commons CLI should filter bad options");
                System.exit(1);
            }

        } catch (ParseException e) {
			outputConsole.println( "Bad invocation: " + e.getMessage() );
        	HelpFormatter formatter = new HelpFormatter();
        	formatter.printHelp(outputConsole, 80, "mergerCLI", "--------",opts,5,3,"--------",true );
        } catch (InvalidParameterException e) {
            outputConsole.println("Invalid Parameter: " + e);
        } catch (BadFileException e) {
            outputConsole.println("Input file not valid: " + e);
        } catch (IOException e) {
            outputConsole.println("Input / Output problem: " + e);
        }
	}

	
	private static Map<String,String> smwGetInput() {
	  Scanner sc = new Scanner(System.in);	  
	  Map<String,String> data = new HashMap();  
	  
	  System.out.print("Semantic Media Wiki User: ");
	  String b = sc.nextLine();
	  data.put("user",b);
	  
	  System.out.print("Semantic Media Wiki Password: ");
	  //String c = new String(System.console().readPassword());
	  String c = sc.nextLine();	   
	  data.put("pass",c);
	  
	  return data;
	}
	
	
    private static void slomMarshall(SLOM theSlom, Serializer sOut, String pathOut) throws InvalidParameterException, IOException {

            ByteArrayOutputStream baos1 = new ByteArrayOutputStream();
            ByteArrayOutputStream baos2 = new ByteArrayOutputStream();

            LinkedList<OutputStream> soList =  new LinkedList();

            soList.add(baos1);
            soList.add(baos2);

            sOut.marshall(theSlom,soList.iterator());

            FileOutputStream fos = new FileOutputStream(pathOut);
            ZipOutputStream zos = new ZipOutputStream(fos);
            ZipEntry cmze = new ZipEntry(SLOMUtil.INTUI_CM_ZIPENTRY);
            ZipEntry ccmze = new ZipEntry(SLOMUtil.INTUI_CCM_ZIPENTRY);

            zos.putNextEntry(cmze);
            zos.write(baos1.toByteArray());

            zos.putNextEntry(ccmze);
            zos.write(baos2.toByteArray());

            zos.close();
    }

    private static void imsldMarshall(SLOM theSlom, Serializer sOut, String pathOut) {
            try {
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                LinkedList<OutputStream> soList =  new LinkedList();
                soList.add(baos);
                
                sOut.marshall(theSlom,soList.iterator());            
            
                FileOutputStream fos = new FileOutputStream(pathOut);
                ZipOutputStream zos = new ZipOutputStream(fos);
                ZipEntry ze = new ZipEntry("imsmanifest.xml");
                
                zos.putNextEntry(ze);
                zos.write(baos.toByteArray());
                
                zos.close();
            } catch (Exception e) {
                System.err.println("There was a problem while marshalling IMS LD content");
                e.printStackTrace();
            }
    }
    
    
    private static void scormMarshall(SLOM theSlom, Serializer sOut, String pathOut) {
            try {
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                LinkedList<OutputStream> soList =  new LinkedList();
                soList.add(baos);
                
                sOut.marshall(theSlom,soList.iterator());            
            
                FileOutputStream fos = new FileOutputStream(pathOut);
                ZipOutputStream zos = new ZipOutputStream(fos);
                ZipEntry ze = new ZipEntry("imsmanifest.xml");
                
                zos.putNextEntry(ze);
                zos.write(baos.toByteArray());
                
                zos.close();            

            } catch (Exception e) {
                System.err.println("There was a problem while marshalling SCORM content");
                e.printStackTrace();
            }
    }
}
