/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.merger;

import java.io.File;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.util.Set;
import java.util.Iterator;
import org.intuitel.slom.SLOM;

/**
 * Implements SLOM serialization.
 *
 * <p>The SLOMSerializer read metadata from a SLOM package and translates it to its internal 
 * SLOM representation.</p>
 *
 *
 * <p>Further information of the transformation details can be found at the 
 * <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.2</p>
 * 
 * @see Serializer
 * @see SerializerFactory
 * @see org.intuitel.slom.SLOM
 */
public class SLOMSerializer extends Serializer {

    /**
    * Reads CM and CCM from respective InputStream objects and creates a SLOM object.
    * 
    * <p>This unmarshalling method is exclusive of the SLOM format. It allows a more efficient
    * usage or resources, since it is expected that software using (i.e. editing or reading) 
    * SLOM are more likely to store the CM and CCM rather than the SLOM package itself.</p> 
    * 
    * @param cm The Cognitive Model of the SLOM course
    * @param ccm The Content Cognitive Model of the SLOM course    
    * @return A SLOM object containing the information read from the input
    */
    public SLOM unmarshall (InputStream cm, InputStream ccm) throws BadFileException, IOException{
        SLOM theSlom = new SLOM(cm, ccm);
        return theSlom;
    }

    
    /** 
    * Reads content from a file and returns a SLOM object with the information read.
    *
    * <p>The unmarshall method reads content from a file and returns a SLOM object with
    * the information read.</p>
    *
    * <p>The method accepts a Set of File objects. In the SLOM case, the Set is expected
    * to contain a single zip file with the SLOM package.</p>
    */
    public SLOM unmarshall (Set<File> inputCourse) throws BadFileException, IOException{
//    A Set of File objects if more than one is needed
        //The first File in the set is expected to be the SLOM package

        Iterator<File> it = inputCourse.iterator();

        if (!it.hasNext())
        {
            throw new BadFileException("Empty Set while parsing SLOM package."); 
        }

        //the first element is enough
        File f = it.next();

        SLOM theSlom = new SLOM(f);

        return theSlom;
    }
    
    /** 
    * Writes the information stored in a SLOM object in a OutputStream.
    *
    * <p>The marshall method serializes a SLOM object in the OWL/RDF representation</p>
    *
    * <p>The method accepts the SLOM object to be serialized, and a Iterator of 
    * OutputStream objects. Two OutputStream are required, to store the CM and the CCM 
    * respectively.</p>
    *   
    * <p>The method returns nothing, since the generated information is written in
    * the OutputStream objects passed as parameter.</p>
    *    
    */
    public void marshall (SLOM theSlom, Iterator<OutputStream> it) throws InvalidParameterException {

        //important: the client code should guarantee the order of the iterator
        //           because the CM will be written to the first OutputStream, 
        //           and the CCM to the second

        OutputStream os = null;
        //only two iterations, no loop required

        //the first OutputStream is filled with the CM
        if (it.hasNext()) {
            os = it.next();
            theSlom.writeCM(os);
        } else {
            throw new InvalidParameterException("For SLOM marshalling, two OutputStream must be provided");
        }

        // the second OutputStream is filled with the CCM
        if (it.hasNext()) {
            os = it.next();
            theSlom.writeCCM(os);
        } else {
            throw new InvalidParameterException("For SLOM marshalling, two OutputStream must be provided");
        }
    }

    /**
    * Determines if the received file contains a SLOM package.
    *
    * <p>The decision is based on the use of the <i>isValidSLOM</i> method provided
    * by <i>SLOM</i> objects.</p>
    *
    * @param SLOM_file The SLOM  package to validate.    
    * @return a String with the ERROR code resulting from the validation
    *
    * @see org.intuitel.slom.SLOM
    * @see org.intuitel.slom.SLOMError    
    */
    public String isValidSLOM (File SLOM_file) throws IOException {
//    The return will be in the form of ERR_code;ERR_description
        return SLOM.isValidSLOM(SLOM_file);
    }
}

