/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.merger;

import java.io.BufferedWriter;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;
import java.io.IOException;
import java.io.FileWriter;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Set;
import java.util.HashSet;
import java.nio.charset.Charset;
import java.io.FileOutputStream;
import java.util.zip.ZipOutputStream;
import java.util.zip.ZipEntry;
import java.io.FileNotFoundException;
import java.net.URL;
import java.net.MalformedURLException;

import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.HtmlForm;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.xml.XmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlSubmitInput;
import com.gargoylesoftware.htmlunit.html.HtmlTextInput;
import com.gargoylesoftware.htmlunit.util.Cookie;

import org.intuitel.slom.SLOMUtil;
import org.apache.commons.io.IOUtils;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;


/**
 * Retrieves RDF description of contents in a Semantic Media Wiki server.
 *
 * <p>SMWScrapper objects use web scraping techniques (provided by the WebClient library) 
 * to download OWL/RDF descriptions of a Semantic Media Wiki course.</p>
 *
 * <p>The server that stores the Semantic Media Wiki course should be able
 * to export courses acording the INTUITEL guidelines (see INTUITEL project 
 * deliverable 4.2 for more information about this format).</p>
 *
 * <p>The scrapper navigates through the SMW course and retrieves the OWL/RDF descriptions, 
 * organizing them as KnowledgeDomain, ConceptContainer and KnowledgeObject. 
 * In normal operation, the following code can be used to launch the Scrapper: </p>
 * 
 * <pre>
   SMWScrapper scrapper = null;
   try {
     scrapper = new SMWScrapper(pathIn, data.get("user"), data.get("pass"));
   } catch (MalformedURLException e) {
     System.err.println("The provided URL is not valid.");
     //(..)
   }
   String path = scrapper.doScrap();
 * </pre>
 * <p>Being <i>path</i> the route in the local file system where the downloaded content
 * has been stored</p>.
 * @see SMWSerializer
 * @see com.gargoylesoftware.htmlunit.WebClient
 */
public class SMWScrapper {
     
    WebClient webClient;
    String url;
    String initialKD;
    String username;
    String password;
    
    /**
    * Creates the Scrapper.
    *
    * <p>Creates the SMWScrapper using the <i>url</i>, <i>user</i> and <i>password</i> passed
    * as parameters.</p>
    *
    * <p>Users should mind that the URL point to the course, not to the server root. For example: </p>
    * <pre>
    http://kalmar30.fzi.de/index.php/KdNetworkDesign
    * </pre>
    * <p>instead of:</p>
    * <pre>
    http://kalmar30.fzi.de/
    * </pre>    
    * @param url the URL of the course at the SMW server.
    * @param user the username at the SMW server.
    * @param pass the user password at the SMW server.
    */
    public SMWScrapper(String url, String user, String pass) throws MalformedURLException {
        webClient = new WebClient();                
	URL theURL= new URL(url);
	this.url=theURL.getProtocol()+"://"+theURL.getHost();
	this.initialKD = theURL.getFile().substring(theURL.getFile().lastIndexOf("/")+1);
	this.username=user;
	this.password=pass;
	  
	configureWebClient();
    }
    
    private void configureWebClient() {
    
        try {               
        
	  // Get the first page
	  HtmlPage page1 = (HtmlPage) webClient.getPage(url + "/index.php/Spezial:Anmelden");

	  // Get the form that we are dealing with and within that form,
	  // find the submit button and the field that we want to change.
	  HtmlForm form = page1.getFormByName("userlogin");

	  // Enter login and passwd
	  form.getInputByName("wpName").setValueAttribute(username);
	  form.getInputByName("wpPassword").setValueAttribute(password);
      
	  // Click "Sign In" button/link
	  page1 = (HtmlPage) form.getInputByName("wpLoginAttempt").click();

	  // I added the cookie section but this returns a null pointer exception    
	  Set<Cookie> cookie = webClient.getCookieManager().getCookies();

	  if(cookie != null){
	      Iterator<Cookie> i = cookie.iterator();
	      while (i.hasNext()) {
		  webClient.getCookieManager().addCookie(i.next());
	      }
	  }
        } catch (IOException e) {
            System.err.println("Unable to configure WebClient object.");
            e.printStackTrace();
            System.exit(1);
        }
    }
  
    /**
    * Retrieves de OWL/RDF files.
    *
    * <p>When executed in a SMWSerializer object configured with <i>url</i>, <i>user</i> and <i>password</i>, 
    * this method browses the configured URL, taking the following steps: </p>
    *
    * <ol>
    * <li>Retrieve the initial URL content and log in the server</li>
    * <li>Stores the received cookie (creates logged session)</li>
    * <li>Retrieves OWL/RDL description of the initial page (the KnowledgeDomain)</li>
    * <li>Parses the retrieved description and get the URL of ConceptContainers</li>
    * <li>Retrieves de OWL/RDF descriptions of the ConceptContainers</li>
    * <li>Parses the CCs and get the URL of KnowledgeObjects</li>
    * <li>Retrieves de OWL/RDF descriptions of the KnowledgeObjects</li>
    * <li>Creates a manifest file describing all the retrieved files</li>    
    * <li>Packages the OWL/RDF descriptions and the manifests within a ZIP file</li>        
    * </ol>    
    *
    * <p>This ZIP file can be later used by the SMWSerializer to create the CM and CCM files</p>
    *
    * @see SMWSerializer
    * @return the path in the local file system of the created ZIP package
    */
    public String doScrap() {
      String coursePath = null;      
      Set<String> CCs = new HashSet();
      Set<String> KOs = new HashSet();
      Set<String> CCFilePaths = new HashSet();
      Set<String> KOFilePaths = new HashSet();
      
      //Build zip file and manifest
      String tDir = System.getProperty("java.io.tmpdir");
      String smwManifestPath = tDir + "/smwManifest";
      String smwZipPath =  tDir + "/smwZipContent";
      FileOutputStream fos = null;      
      ZipOutputStream zos = null;
      PrintWriter outManifest = null;
      
      try {	  
	outManifest = new PrintWriter(smwManifestPath);
	fos = new FileOutputStream(smwZipPath);
	zos = new ZipOutputStream(fos);
      } catch (IOException e) {
	System.err.println("Unable to create smwManifest while retrieving SMW info");
	e.printStackTrace();
      } catch (Exception e) {
	System.err.println("Unable to create SMW Zip File.");
	e.printStackTrace();
      }
      
      //add KD IRI to the manifest
      outManifest.println("KDIRI,"+url+"/"+initialKD);
            
      //retrieve KD file, which is the root of the course
      String KDFilePath = retrieveRDFContent(initialKD);
      
      //add KD to the manifest and zipfile
      outManifest.println("CM,"+new File(KDFilePath).getName());
      addToZIP(KDFilePath,zos);
                     
      //parse KD and find CCs
      try {
	DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	DocumentBuilder db = dbf.newDocumentBuilder();
	Document document = db.parse(new File(KDFilePath));
	NodeList nodeList = document.getElementsByTagName("intui:containsConceptContainer");
	
	for(int x=0,size= nodeList.getLength(); x<size; x++) {
	  String CCURL = nodeList.item(x).getAttributes().getNamedItem("rdf:resource").getNodeValue();
	  CCs.add(CCURL.substring(CCURL.lastIndexOf("/")+1));
	}
	
      } catch (Exception e) {
	System.err.println("Unable to parse retrieved RDF file");
	e.printStackTrace();
      }
      
      //iterate CCs
      for (String s: CCs) {
      
	String CCFilePath = retrieveRDFContent(s);
	
	//add retrieved document to the manifest
	outManifest.println("CM,"+new File(CCFilePath).getName());
	
	//add retrieved document to the Zip file
	addToZIP(CCFilePath,zos);
	
	//parse and find KOs
	try {
	  KOs.clear();
	  DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	  DocumentBuilder db = dbf.newDocumentBuilder();
	  Document document = db.parse(new File(CCFilePath));
	  NodeList nodeList = document.getElementsByTagName("intui:ContainsKnowledgeObject");
	  
	  for(int x=0,size= nodeList.getLength(); x<size; x++) {
	    String KOURL = nodeList.item(x).getAttributes().getNamedItem("rdf:resource").getNodeValue();
	    KOs.add(KOURL.substring(KOURL.lastIndexOf("/")+1));
	  }
	  
	} catch (Exception e) {
	  System.err.println("Unable to parse retrieved RDF file");
	  e.printStackTrace();
	}	
	
	//iterate KOs
	for (String k: KOs) {
	  String KOFilePath = retrieveRDFContent(k);
	  
	  //add retrieved document to the manifest
	  outManifest.println("CCM,"+new File(KOFilePath).getName());
	
	  //add retrieved document to the Zip file
	  addToZIP(KOFilePath,zos);
	  
	  //delete temporary file
	  new File(KOFilePath).delete();	  
	}
	
	//delete temporary file
	new File(CCFilePath).delete();
      }
           
      //all files added to manifest and zip. 
      //Add manifest to zip and close both
      try {
	outManifest.close();
	addToZIP(smwManifestPath,zos);
	zos.close();
	
	//delete temporary files
	new File(KDFilePath).delete();
	new File(smwManifestPath).delete();
	
      } catch (IOException e) {
	System.err.println("General Input/Output problem while closing zipfile.");
	e.printStackTrace();
      }
      
      return smwZipPath;
    }
    
    private void addToZIP(String pathToAdd, ZipOutputStream zos) {
      try {
	File fi = new File(pathToAdd);
	FileInputStream fis = new FileInputStream(fi);
	byte[] bytes = new byte[1024];
	int length;

	zos.putNextEntry(new ZipEntry(fi.getName()));     
	while ((length = fis.read(bytes)) >= 0) {
	  zos.write(bytes, 0, length);
	}
	zos.closeEntry();
	fis.close();
      } catch (Exception e) {
	System.err.println("Unable to add entry to SMW zip file");
	e.printStackTrace();
      }
    }

    /**
    * Retrieves a set of OWL/RDF files from the SMW server.
    *
    * <p>Uses the Scrapper to retrieve a set of OWL/RDF files from 
    * the SMW server, which corresponds to the set of pageNames 
    * given at the input param <i>paths</i>
    * 
    * This method is a wrapper of <i>retrieveRDFContent</i>.
    *
    * @param paths A set with the name of the pages to retrieve. That is, the last part of the URL
    * @return A set of paths in the local file system of the retrieved OWL/RDF description
    * @see #retrieveRDFContent(String pageName)
    */  
    public Set<String> retrieveRDFContentSet(Set<String> paths){
    
      Set filePaths = new HashSet();
      for (String s:paths){
           String p = retrieveRDFContent(s);
           filePaths.add(p);                 
      }    
      return filePaths;
    }

    /**
    * Retrieves one OWL/RDF file from the SMW server.
    *
    * <p>Uses the Scrapper to retrieve the OWL/RDF file of the course 
    * page given by the URL passed as parameter. This method uses a 
    * special feature of Semantic Media Wiki servers, created by 
    * the INTUITEL project, that provides OWL/RDF description of course
    * contents acording to SLOM nomenclature (i.e. according to the 
    * Pedagogical Ontology given by <a href="http://www.intuitel.eu/public/2014/03/intui_PO.owl">http://www.intuitel.eu/public/2014/03/intui_PO.owl</a>.</p>
    *
    * <p>For example, to retrieve the OWL/RDF content of the following URL:</p>
    * <pre>http://kalmar30.fzi.de/index.php/KdNetworkDesign.</pre>
    *
    * <p>The Scrapper creates and uses the following URL:</p>
    * <pre>http://kalmar30.fzi.de/index.php/Spezial:RDF_exportieren/KdNetworkDesign.</pre>
    *
    * @param pageName The name of the page to retrieve. That is, the last part of the URL
    * @return The path in the local file system of the retrieved OWL/RDF description
    */  
    public String retrieveRDFContent(String pageName) {
    
        String tmpPath = null;
         try{
	  XmlPage page = (XmlPage) webClient.getPage(url+"/index.php/Spezial:RDF_exportieren/"+pageName);
	  //  Get page as XML/RDF
	  String htmlBody = page.getWebResponse().getContentAsString();
	  //  Save the response in a file
	  String tDir = System.getProperty("java.io.tmpdir");
	  tmpPath = tDir + "/" + pageName;


	  
	  //Replace SMW specific IRI with SLOM IRI	  
// 	  String newIRI = SLOMUtil.INTUI_PO.toString()+"/";
// 	  String oldIRI1, oldIRI2, oldIRI3;
// 	  
// 	  oldIRI1 = url + "/index.php/Spezial:URI-Aufl%C3%B6ser/";
// 	  oldIRI2 = url + "/index.php/Spezial:URI-Aufl&#37;C3&#37;B6ser/";
// 	  oldIRI3 = url + "/index.php/Spezial:RDF_exportieren/";
// 	  
// 	  Set<String> oldIRIs = new HashSet();
// 	  oldIRIs.add(oldIRI1);
// 	  oldIRIs.add(oldIRI2);
// 	  oldIRIs.add(oldIRI3);
// 	  htmlBody = changeIRIinString(oldIRIs, newIRI, htmlBody);  
// 
 	  BufferedWriter bw = new BufferedWriter(new FileWriter(new File(tmpPath)));
 	  bw.write(htmlBody);
 	  bw.close();
/*	  
	  //changeIRIinFile(oldIRI, newIRI, tmpPath);	  */
	  
         } catch(IOException e) {       
 	  e.printStackTrace();
 	  System.exit(1);
         }
        
        return tmpPath;
    }
    
    private void changeIRIinFile(Set<String> oldIRIs, String newIRI, String filePath) {
    
      try {
	File file = new File(filePath);
	BufferedReader reader = new BufferedReader(new FileReader(file));
	String line = "", content = "";
	while((line = reader.readLine()) != null)
      {
	content += line + "\r\n";
      }
      reader.close();

      for (String s:oldIRIs) {
	content = content.replaceAll(s, newIRI);      
      }
      
      FileWriter writer = new FileWriter(filePath);
      writer.write(content);
      writer.close();      
      } catch (FileNotFoundException e) {
	    System.err.println("Unable to read temporal file");
	    e.printStackTrace();
	    System.exit(1);      
      }  catch (IOException e) {
	    System.err.println("General Input Output problem");
	    e.printStackTrace();
	    System.exit(1);            
      }
    }    
    
    private String changeIRIinString(Set<String> oldIRIs, String newIRI, String content) {
    
      String a= "";
      for (String s:oldIRIs) {
	content = content.replaceAll(s, newIRI);
	a = a + "_";
      }
      return content;

    }   
}
