/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.merger;

import java.io.File;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import java.util.HashSet;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.Set;
import java.util.Iterator;

import org.semanticweb.owlapi.model.*;
import org.semanticweb.owlapi.io.*;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.util.OWLOntologyMerger;

import org.intuitel.slom.SLOM;
import org.intuitel.slom.SLOMUtil;

import org.apache.commons.io.IOUtils;


/**
 * Implements Semantic Media Wiki serialization.
 *
 *
 * <p>The SMWSerializer read content initially created as a course in a
 * Semantic Media Wiki server, and translates it to its SLOM representation.</p>
 *
 * <p>In the case of Semantic Media Wiki marshalling is not supported. This is 
 * because the SMW course model does not define the concept of <i>course package</i> 
 * that could be imported to deploy a course. Instead, Semantic Media Wiki courses 
 * require human intervention for their creation. </p>
 *
 * <p>Further information of the transformation details can be found at the 
 * <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.2</p>
 * 
 * @see Serializer
 * @see SerializerFactory
 * @see SMWScrapper
 */
public class SMWSerializer extends Serializer {

    /**
    * Reads content from a file and returns a SLOM object with the information read.
    *
    * <p>The unmarshall method reads content from a file and returns a SLOM object with
    * the information read. In the case of Semantic Media Wiki transformations, the file
    * is build by scraping the web course content. This is done by using 
    * <i>SMWScrapper</i> objects</p>
    *
    * 
    * @param inputCourse A Set of files containing the course information in the desired format
    * @return A SLOM object containing the information read from the input
    *
    * @see SMWScrapper
    *
    **/
    public SLOM unmarshall (Set<File> inputCourse) throws BadFileException, IOException{
      //The first file in the Set is a ZIP file with all the required info           
      ZipFile zfile = null;
      ZipEntry manifestEntry = null;
      InputStream manifestIn1 = null;
      InputStream manifestIn2 = null;        
      InputStream manifestIn3 = null;
      
      //get the File object, assuming that it is a zip file (imsld package)
      Iterator<File> it = inputCourse.iterator();
      if (!it.hasNext())
      {
	  throw new BadFileException("Empty Set while parsing " + getExternalFormatName() + " package."); 
      }
      File f = it.next();
      
      //Inside the zip, the smwManifest file describes which 
      //files goes to the CM and CCM
      
      //get the imsmanifest.xml inputStream
        try {
	  zfile = new ZipFile(f);
      
	  manifestEntry = zfile.getEntry(SLOMUtil.SMW_MANIFEST_ZIPENTRY);
	  manifestIn1 = zfile.getInputStream(manifestEntry);
	  manifestIn2 = zfile.getInputStream(manifestEntry);
	  manifestIn3 = zfile.getInputStream(manifestEntry);
        } catch (ZipException e) {
            System.err.println("Unable to open " + getExternalFormatName() + " package as ZIP file.");
        } 
                
        Set<String> CMFilePaths = getPaths(manifestIn1, "CM");
        Set<String> CCMFilePaths = getPaths(manifestIn2, "CCM");
   
        String KDIRI = getPaths(manifestIn3, "KDIRI").iterator().next();
        
        OWLOntology mergedCM = mergeOntologies(CMFilePaths,zfile,KDIRI);
        OWLOntology mergedCCM = mergeOntologies(CCMFilePaths,zfile,KDIRI);        

	ByteArrayOutputStream baosCM = new ByteArrayOutputStream();
	ByteArrayOutputStream baosCCM = new ByteArrayOutputStream();
        // Save to RDF/XML
        try {
	  OWLOntologyManager man = OWLManager.createOWLOntologyManager();
	  
	  man.saveOntology(mergedCM, new RDFXMLOntologyFormat(),baosCM);
	  man.saveOntology(mergedCCM, new RDFXMLOntologyFormat(),baosCCM);
	    
	} catch (OWLOntologyStorageException e) {
	  System.err.println("Unable to merge Ontologies, problem while storing the merged ontology");
	  e.printStackTrace();
	  System.exit(1);
	}
	
	InputStream cmIn = new ByteArrayInputStream(baosCM.toByteArray());
	InputStream ccmIn = new ByteArrayInputStream(baosCCM.toByteArray());
	
	SLOM theSlom = new SLOM(cmIn, ccmIn);
        return theSlom;                       
    }


     private OWLOntology mergeOntologies(Set<String> filePaths, ZipFile zfile, String KDIRI) {
        OWLOntology merged = null;
	try {
	  // Just load two arbitrary ontologies for the purposes of this example
	  OWLOntologyManager man = OWLManager.createOWLOntologyManager();
	  
	  for (String s: filePaths) {        
	    ZipEntry entry = zfile.getEntry(s);
	    InputStream inEntry = zfile.getInputStream(entry);
	    
	    man.loadOntologyFromOntologyDocument(inEntry);	    	    
	  }	  
	  
	  OWLOntologyMerger merger = new OWLOntologyMerger(man);
	  IRI mergedOntologyIRI = IRI.create(KDIRI);                
	  merged = merger.createMergedOntology(man, mergedOntologyIRI);
	} catch (OWLOntologyCreationException e) {
	  System.err.println("Unable to merge Ontologies, problem while creating the merged ontology");
	  e.printStackTrace();
	  System.exit(1);	
	} catch (IOException e) {
	  System.err.println("Unable to merge Ontologies, general Input/Output problem");
	  e.printStackTrace();
	  System.exit(1);
	}
    
      return merged;
    }
    /**
    * Marshall is not allowed for SMW course format.
    *
    * <p>Further information of the transformation details can be found at the 
    * <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.2</p>
    */ 
    public void marshall (SLOM theSlom, Iterator<OutputStream> it) throws InvalidParameterException {
        System.err.println("Semantinc Media Wiki Marshalling is not supported.\n See the API documentation for details.");
    }

    /**
    * Returns "Semantic Media Wiki".
    * 
    * @return "Semantic Media Wiki"
    */
    public String getExternalFormatName(){
      return SLOMUtil.SMW_NAME;
    }
    
    private Set<String> getPaths(InputStream in, String filterType) throws IOException{
    
      Set<String> output = new HashSet();
      BufferedReader bf = new BufferedReader(new InputStreamReader(in));
      String line = null;
      String entryType = null;
      String entry = null;
      
      while((line = bf.readLine()) != null) {
	//decide if we include this line or not
	 entryType = line.substring(0,line.lastIndexOf(","));
	 entry = line.substring(line.lastIndexOf(",") + 1);
	  
	 if (filterType!=null && filterType.equals(entryType)) {
	  //include in the output set
	  output.add(entry);	  
	 }
      }
      return output;
    }    
    
}

