/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.merger;

import java.io.File;
import java.util.Set;
import java.io.OutputStream;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import org.apache.commons.io.IOUtils;

import org.intuitel.slom.SLOM;
import org.intuitel.slom.SLOMUtil;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.net.URL;
import org.intuitel.merger.BadFileException;


import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;

/**
 * Abstract class that servers as wrapper for those serializations based on 
 * XSLT transformations.
 *
 * <p>This class was just included to reuse code in those cases in which
 * a XSLT stylesheet is used for the transformation. In those cases, most
 * of the code is written here, while the Subclasses set the value of
 * specific parameters.<p>
 *
 * <p>The use of XSLTSerializer asumes the following conditions for 
 * the transformation:</p>
 *
 * <ul>
 * <li>The transformation from the external format to SLOM is based on 
 * two stylesheets: one creates the CM and the other one creates the CCM</li>
 * <li>The transformation from SLOM to the external format is based on 
 * the application of a single stylesheet.</li>
 * </ul> 
 *
 * <p>With such asumptions, this class provides the marshall and unmarshall 
 * methods while the extending classes should provide the XSLT transformation files.</p>
 */
public abstract class XSLTSerializer extends Serializer {

    /** 
    * Returns the path to the XSLT stylesheet file that will be used to generate the CCM.
    *
    * <p>This method, implemented in the subclasses, returns a String with the 
    * path of the XSLT file that will be used to transform from the input format
    * to build the CCM file.</p>
    * 
    * <p>In the current implementation, this method is used by unmarshall and marshall 
    *  methods. The returned path refers to the location of the XSLT file in the jar 
    * package of the library.</p>
    * 
    * @return A String with the path to the XSLT file that will do the transformation
    */
    protected abstract String getCCMXSLT();


    /** 
    * Returns the path to the XSLT stylesheet file that will be used to generate the CM.
    *
    * <p>This method, implemented in the subclasses, returns a String with the 
    * path of the XSLT file that will be used to transform from the input format
    * to build the CM file.</p>
    * 
    * <p>In the current implementation, this method is used by unmarshall and marshall 
    *  methods. The returned path refers to the location of the XSLT file in the jar 
    * package of the library.</p>
    * 
    * @return A String with the path to the XSLT file that will do the transformation
    */
    protected abstract String getCMXSLT();


    /** 
    * Returns the path to the XSLT stylesheet file that will be used to produce the external format.
    *
    * <p>This method, implemented in the subclasses, returns a String with the 
    * path of the XSLT file that will be used to transform a SLOM file to the selected 
    * external format.</p>
    * 
    * <p>In the current implementation, this method is used by unmarshall and marshall 
    *  methods. The returned path refers to the location of the XSLT file in the jar 
    * package of the library.</p>
    * 
    * @return A String with the path to the XSLT file that will do the transformation
    */
    protected abstract String getToExternalFormatXSLT();

    
    /** 
    * Returns the name of the external format produced by the XSLTSerializer.
    *
    * <p>This method, implemented in the subclasses, returns a String with the 
    * name of the external format to be produced.</p>
    * 
    * <p>In the current implementation, this method is used by unmarshall and marshall 
    *  methods.</p>
    * 
    * @return A String with the name of the external format produced by the XSLTSerializer
    */
    protected abstract String getExternalFormatName();

    public SLOM unmarshall (Set<File> inputCourse) throws BadFileException, IOException {
//    A Set of File objects if more than one is needed.
        ZipFile zfile = null;
        ZipEntry manifestEntry = null;
        InputStream manifestIn1 = null;
        InputStream manifestIn2 = null;
        ByteArrayOutputStream baosCM = new ByteArrayOutputStream();
        ByteArrayOutputStream baosCCM = new ByteArrayOutputStream();


        //get the File object, assuming that it is a zip file (imsld package)
        Iterator<File> it = inputCourse.iterator();
        if (!it.hasNext())
        {
            throw new BadFileException("Empty Set while parsing " + getExternalFormatName() + " package."); 
        }
        File f = it.next();

        //get the imsmanifest.xml inputStream
        try {
            zfile = new ZipFile(f);
        } catch (ZipException e) {
            System.err.println("Unable to open " + getExternalFormatName() + " package as ZIP file.");
        }

        manifestEntry = zfile.getEntry(SLOMUtil.IMSCP_MANIFEST_ZIPENTRY);
        manifestIn1 = zfile.getInputStream(manifestEntry);
        manifestIn2 = zfile.getInputStream(manifestEntry);

        //use XSLT to translate to SLOM. Get two OutputStream (cm and ccm)
        InputStream xsltCmIn = SLOM.class.getResourceAsStream(getCMXSLT());
        InputStream xsltCcmIn = SLOM.class.getResourceAsStream(getCCMXSLT());

        transformFile(xsltCmIn,manifestIn1,baosCM);
        transformFile(xsltCcmIn,manifestIn2,baosCCM);

        //create the SLOM object
        InputStream cm = new ByteArrayInputStream(baosCM.toByteArray());
        InputStream ccm = new ByteArrayInputStream(baosCCM.toByteArray());

        SLOM theSlom = new SLOM(cm, ccm);
        return theSlom;
    }


    /** 
    * Transforms a XML file acording to the rules provided by a XSLT stylesheet.
    *
    * <p>This method is internally used by the marshall and unmarshall methods, but has
    * been set as protected (instead of private) to allow developers who want to extend
    * thi class to reuse the provided functionality. </p>
    * 
    * @param stylesheet The XSLT to apply in the transformation
    * @param input The XML file to transform
    * @param output An OutputStream to write the result of the transformation     
    * @see #marshall
    * @see #unmarshall    
    */
    protected static void transformFile(InputStream stylesheet, InputStream input, OutputStream output){
        try{

            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document doc = builder.parse(input);

            TransformerFactory tFactory = TransformerFactory.newInstance();                       
            StreamSource stylesource = new StreamSource(stylesheet);
            Transformer transformer = tFactory.newTransformer(stylesource);

            if (transformer == null) {            
                System.err.println("Transformation failed, transformer object was NULL");
                System.exit(1);
            }
            transformer.setOutputProperty(OutputKeys.INDENT,"yes");

            DOMSource dom = new DOMSource(doc);
            StreamResult result = new StreamResult(output);

            transformer.transform(dom,result);
        }
        catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (TransformerConfigurationException e) {
            e.printStackTrace();
        } catch (TransformerException e) {
            e.printStackTrace();
        }
    }
    
    /** 
    * Merges two XML files containing CM and CCM into a single file.
    *
    * <p>This method is internally used by the marshall method, in order to execute the 
    * transformation to the external format taking a single file as input source. It has
    * been set as protected (instead of private) to allow developers who want to extend
    * thi class to reuse the provided functionality. </p>
    * 
    * @param stylesheet The XSLT that actually mergers the files
    * @param input The CM content
    * @param intuiCMLocation the path to the CM file in the local file system
    * @param intuiCCMLocation the path to the CCM file in the local file system    
    * @param output An OutputStream to write the result of the merge 
    *
    * @see #marshall    
    */
    protected static void mergeCMandCCM(InputStream stylesheet, InputStream input, OutputStream output, String intuiCMLocation, String intuiCCMLocation) { 

        try{

            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document doc = builder.parse(input);

            TransformerFactory tFactory = TransformerFactory.newInstance();
            StreamSource stylesource = new StreamSource(stylesheet);
            Transformer transformer = tFactory.newTransformer(stylesource);

            transformer.setOutputProperty(OutputKeys.INDENT,"yes");
            //set input files location relative to the transform file
            transformer.setParameter("CMparam",intuiCMLocation);
            transformer.setParameter("CCMparam",intuiCCMLocation);

            DOMSource dom = new DOMSource(doc);
            StreamResult result = new StreamResult(output);

            transformer.transform(dom,result);
        }
        catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (TransformerConfigurationException e) {
            e.printStackTrace();
        } catch (TransformerException e) {
            e.printStackTrace();
        }
    }

    public void marshall (SLOM theSlom, Iterator<OutputStream> it) throws InvalidParameterException, IOException {
    
       //first step: serialize as SLOM. 
       //            The result is written in a pair of 
       //            FileOutputStreams (cm + ccm) that will be used as temp files

       String cmFilename = SLOMUtil.INTUI_CM_FILE;
       String ccmFilename = SLOMUtil.INTUI_CCM_FILE;
       String tDir = System.getProperty("java.io.tmpdir");
       String tmpCMPath = tDir + "/" + cmFilename;
       String tmpCCMPath = tDir + "/" + ccmFilename;

       FileOutputStream fosCM = new FileOutputStream(tmpCMPath);
       FileOutputStream fosCCM = new FileOutputStream(tmpCCMPath);

       theSlom.writeCM(fosCM);
       fosCM.close();
       theSlom.writeCCM(fosCCM);
       fosCCM.close();


       //I should study if JENA is useful as converter
//       Model modelCM = ModelFactory.createDefaultModel();
//       modelCM.read(new FileInputStream(tmpCMPath),"", "RDF/XML");
//       FileOutputStream fosCM2 = new FileOutputStream(tmpCMPath);
//       modelCM.write(fosCM2, "RDF/XML-ABBREV");
//       fosCM2.close();
//       Model modelCCM = ModelFactory.createDefaultModel();
//       modelCCM.read(new FileInputStream(tmpCCMPath),"", "RDF/XML");
//       FileOutputStream fosCCM2 = new FileOutputStream(tmpCCMPath);
//       modelCCM.write(fosCCM2, "RDF/XML-ABBREV");
//       fosCCM2.close();

       //idea: añadir funciones write con parámetro que me permita
       //      elegir el formato de salida
       //      - el formato por defecto sería el que hay ahora mismo,
       //        pero podría elegir y así podré jugar con leer y escribir 
       //        con JENA. Eso sería útil para las transformaciones
       //        a IMSLD y SCORM

       //second step: merge into one single file. 
       //             The result is  written in a outputStream (ByteArrayOutputStream)
       InputStream cmIn = new FileInputStream(tmpCMPath);
       InputStream xsltMerge = SLOM.class.getResourceAsStream(SLOMUtil.MERGE_CM_AND_CCM);
       ByteArrayOutputStream baosMerged = new ByteArrayOutputStream();

       mergeCMandCCM( xsltMerge,cmIn,baosMerged,tmpCMPath,tmpCCMPath);
       baosMerged.close();

       File f1 = new File(tmpCMPath);
       f1.delete();
       File f2 = new File(tmpCCMPath);
       f2.delete();

       //third step: tranform to IMS LD, writes in the first element of 'it' (the input).
       OutputStream output = null;
       if (it.hasNext()) {
           output = it.next();
       } else {
           throw new InvalidParameterException("For "+getExternalFormatName()+" marshalling, at least one OutputStream must be provided");
       }
       InputStream mergedIn = new ByteArrayInputStream(baosMerged.toByteArray());
       InputStream xsltToExternal = SLOM.class.getResourceAsStream(getToExternalFormatXSLT());

       transformFile(xsltToExternal,mergedIn,output);
    }
}

