/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.slom;

import java.util.*;
import java.io.OutputStream;
import java.io.InputStream;


import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.rdf.model.SimpleSelector;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.reasoner.ValidityReport;
import com.hp.hpl.jena.ontology.OntModelSpec;

import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.*;
import org.semanticweb.owlapi.io.*;
import org.semanticweb.owlapi.util.SimpleIRIMapper;

import org.semanticweb.owlapi.reasoner.OWLReasoner;
import org.semanticweb.owlapi.reasoner.OWLReasonerConfiguration;
import org.semanticweb.owlapi.reasoner.OWLReasonerFactory;
import org.semanticweb.owlapi.reasoner.SimpleConfiguration;
import org.semanticweb.owlapi.reasoner.structural.StructuralReasoner;
import org.semanticweb.owlapi.reasoner.structural.StructuralReasonerFactory;
import org.semanticweb.owlapi.model.OWLDataProperty; 
import org.semanticweb.owlapi.model.PrefixManager; 
import org.semanticweb.owlapi.util.DefaultPrefixManager; 
import org.semanticweb.HermiT.Reasoner;


import org.semanticweb.owlapi.reasoner.NodeSet;
import org.semanticweb.owlapi.reasoner.ConsoleProgressMonitor;
import org.apache.commons.io.IOUtils;

import uk.ac.manchester.cs.owl.owlapi.*;



/**
* Models the CognitiveContentMap part of the SLOM format.
*
* <p>The SLOM course description in a SLOM package is provided by two OWL files, 
* representing the CognitiveMap and the CognitiveContentMap. Where: <p>
*
* <ul>
* <li>The Cognitive Map refers to the concepts that are included in a certain 
* Knowledge Domain. Each of this concept is expressed as a ConceptContainer.</li>
* <li>The Cognitive Content Map refers to the actual course material that 
* can be used to teach/learn those concepts. Each of these part of course 
* material are expressed as KnowledgeObject.</li>
* </ul>
*
* <p>This class provides a Java representation of the <b>CognitiveContentMap</b> part, based on the use
* of the OWLOntology object of the OWLAPI library. This Java class 
* is internally used to create the SLOM object, but developers are discouraged to
* directly use this class.</p>
*
* <p>Further information of the SLOM format can be found at the 
* <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
* 
* @author  Luis de-la-Fuente-Valentín
* @since   1.0
*/
public class CognitiveContentMap
{
    private OWLOntologyManager man;
    private OWLOntology owlOntCCM;

    
    /**
    * Returns the OWLOntology object that represents the CCM.
    *
    * <p>The internal representation of the CognitiveContentMap is based on the OWLAPI library. 
    * This method provides access to the OWLOntology object.</p>
    * <p>The OWLOntology object can be used by developers to execute advanced operation
    * on the SLOM content, such as reasoning, complex queries, etc.</p>
    *
    * @return the OWLOntology object that represents the CCM
    */      
    public OWLOntology getOWLOntology() {
        return owlOntCCM;
    }

    /**
    * Prints in the standard output all the axioms in the CCM.
    *
    * <p>Reads all the axioms at the CCM and print them at the standard output.
    * Useful for debuggin purposes</p>    
    */      
    public void printOWLAxioms(){
        Set<OWLAxiom> all = owlOntCCM.getAxioms();
        Iterator<OWLAxiom> it = all.iterator();
        Vector<OWLAxiom> ccs = new Vector();

        OWLAxiom current = null;
        while (it.hasNext()) {
            current = it.next();
            System.out.println("Type: " + current.getAxiomType());
            System.out.println("==");
            if ( current.getAxiomType().toString().equals("AnnotationAssertion")) {
                OWLAnnotationAssertionAxiom a = (OWLAnnotationAssertionAxiom)current;
                System.out.println(a.getProperty());
                System.out.println(a.getSubject());
                System.out.println(a.getValue());
                System.out.println("============================");
            }
        }
    }

    private void createManager(Iterator<SimpleIRIMapper> iriMaps){
       man = OWLManager.createOWLOntologyManager(); 
       while (iriMaps.hasNext()){
            man.addIRIMapper(iriMaps.next());
       }
    }
    

    /**
    * Builds the CCM object from owl content.
    *
    * 
    * <p>The CCM object is build by reading the information from the OWL content
    * passed as parameter.</p>
    *
    * @param ccmIn the CCM as OWL content
    * @param iriMaps a container with a IRI mapping to use while loading the owl content 
    */
    public CognitiveContentMap(InputStream ccmIn, Iterator<SimpleIRIMapper> iriMaps) {

       createManager(iriMaps);

       try {
           owlOntCCM = man.loadOntologyFromOntologyDocument(ccmIn);// throws OWLOntologyCreationException
       } catch (OWLOntologyCreationException e) {
           System.err.println("Problem while reading the ontology with OWL API");
           e.printStackTrace();
           System.exit(1);
       }
    }

    /**
    * Returns the CCM IRI, as read from the owl file
    *
    * @return The CCM IRI, as read from the owl file
    */    
    public IRI getCCMIRI() {
        IRI owlid = owlOntCCM.getOntologyID().getDefaultDocumentIRI();
        return owlid;
    }


    /**
    * Determines if the received file contains a CCM description.
    *     
    * <p>The decision is based on the SLOM format definition, further information can be found at the 
    * <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
    * 
    *
    * @param ccmIn the CCM content to validate
    * @param iriMaps a container with a IRI mapping to use while loading the owl content 
    * @return true if it is valid, false otherwise.        
    */
    public static boolean isValidCCM(InputStream ccmIn, Iterator<SimpleIRIMapper> iriMaps) {
       boolean isLoadable = false;
       boolean containsKO = false;
       boolean isConsistent = false;

       OWLOntologyManager iriManager = OWLManager.createOWLOntologyManager(); 
       while (iriMaps.hasNext()){
            iriManager.addIRIMapper(iriMaps.next());
       }

       OWLOntology testOwlOnt = null;
       try {
           //if throws an exception, then it is not valid
           testOwlOnt = iriManager.loadOntologyFromOntologyDocument(ccmIn);
           isLoadable = true;

            ///////////////////////////////////////////////
            //basic check (quite fool). If contains KOs
            ///////////////////////////////////////////////
           //Create factories that will produce the objects
           OWLReasonerFactory reasonerFactory = new StructuralReasonerFactory();
           OWLDataFactory fac = iriManager.getOWLDataFactory();

           //Get a reasoner, to query the ontology
           OWLReasoner reasoner = reasonerFactory.createReasoner(testOwlOnt);

           //Get CCs
           OWLClass myClass = fac.getOWLClass(IRI.create("http://www.intuitel.eu/public/intui_PO.owl#KnowledgeObject"));
           NodeSet<OWLNamedIndividual> kos = reasoner.getInstances(myClass, false);
           Set<OWLNamedIndividual> kosFlat = kos.getFlattened();

           containsKO = (kosFlat.size() > 0);

           ///////////////////////////////////////////////
           //advanced. Check for consistency with a reasoner
           //////////////////////////////////////////////
           reasoner.precomputeInferences();
           isConsistent = reasoner.isConsistent();

       } catch (OWLOntologyCreationException e) {
           System.err.println("Problem while reading the ontology with OWL API");
           e.printStackTrace();
           isLoadable = false;
       }
        ///////////////////////////////////////////////
        //advanced. Check for consistency with a reasoner
        ///////////////////////////////////////////////

        return (isLoadable && containsKO && isConsistent); // no CC in owl file
    }

    /** 
    * Writes the content of the CCM as OWL/RDF.
    *
    * <p>This method uses a RDFXMLOntologyFormat to serialized the ontology information
    * and write it in the <i>os</i> OutputStream.</p> 
    *
    * @param os The OutputStream where the information will be written
    */   
    public void write(OutputStream os) {
//        ccmModel.write(os);
           try {
               man.saveOntology(owlOntCCM,new RDFXMLOntologyFormat(), os);//throws OWLOntologyStorageException
           }catch (OWLOntologyStorageException e) {
               System.err.println("Problem while writing the ontology with OWL API");
               e.printStackTrace();
               System.exit(1);
           }
    }

    /**
    * Search KnowledgeObjects in this CCM and return a list of them.
    *
    * <p>This method queries the Axioms of the CCM OWLOntology and finds 
    * the KnowledgeObjects. For each KO
    * found, builds the <i>KnowledgeObject</i> Java representation
    * object and adds it into an <i>ArrayList</i>, then returns this ArrayList.</p>
    *
    * <p>This provides a high abstraction level mean to read the content of the CCM.</p>
    * 
    * @return an ArrayList with the KnowledgeObject found in the CCM
    * @see org.intuitel.slom.KnowledgeObject    
    */
    public ArrayList<KnowledgeObject> getKnowledgeObjects() {

        ArrayList<KnowledgeObject> KOs = new ArrayList();

        //Create factories that will produce the objects
        OWLReasonerFactory reasonerFactory = new Reasoner.ReasonerFactory();
        OWLDataFactory fac = man.getOWLDataFactory();

        //Get a reasoner, to query the ontology
        OWLReasoner reasoner = reasonerFactory.createReasoner(owlOntCCM);

        //Get KOs.
        OWLClass myKOclass = fac.getOWLClass(IRI.create("http://www.intuitel.eu/public/intui_PO.owl#KnowledgeObject"));
        NodeSet<OWLNamedIndividual> individualsKONodeSet = reasoner.getInstances(myKOclass, true);
        Set<OWLNamedIndividual> niKOs = individualsKONodeSet.getFlattened();
        Iterator<OWLNamedIndividual> it = niKOs.iterator();
        while (it.hasNext() )
        {
            OWLNamedIndividual tmpKOWL = it.next();
            KnowledgeObject tmpKO=new KnowledgeObject(tmpKOWL.toString());
            KOs.add(tmpKO);
        }
        return KOs;
    }

    private void configureKORelations(Set<KnowledgeObjectRelationship> koRels) {
        Set<OWLAxiom> all = owlOntCCM.getAxioms();
        Iterator<OWLAxiom> itAxioms = all.iterator();

        OWLAxiom currentAxiom = null;
        OWLAnnotationAssertionAxiom currentAAAxiom = null;
        KnowledgeObjectRelationship currentKOR = null;
        String property = null;
        String subject = null;
        String KORName = null;
        while (itAxioms.hasNext()) {
            Iterator<KnowledgeObjectRelationship> itKORels = koRels.iterator();
            currentAxiom = itAxioms.next();
            if ( currentAxiom.getAxiomType().toString().equals("AnnotationAssertion")) {
                currentAAAxiom = (OWLAnnotationAssertionAxiom)currentAxiom;
                subject = currentAAAxiom.getSubject().toString(); 

                while (itKORels.hasNext()) {
                    currentKOR = itKORels.next();
                    KORName = currentKOR.getName();

                    if (KORName.equals("<"+subject+">")) {
                        property = currentAAAxiom.getProperty().toString();
                        if (property.equals("<http://www.intuitel.eu/public/intui_PO.owl#hasPredecessorInMicroLp>")) {
                            currentKOR.setHasPredecessorInMicroLp(currentAAAxiom.getValue().toString());
                        } else if (property.equals("<http://www.intuitel.eu/public/intui_PO.owl#hasSuccessorInMicroLp>")) {
                            currentKOR.setHasSuccessorInMicroLp(currentAAAxiom.getValue().toString());
                        }
                    }

                }
            } 
        }
    }

    /**
    * Configures the KnowledgeObject in the list received as parameter.
    * 
    * <p>For each KnowledgeObject in the received list, queries the Axioms in the 
    * OWLOntology and reads all the properties assigned to the KnowledgeObject.
    * With them, configures the Java representation of the KnowledgeObject.</p>
    *
    * <p>This method is used for the initial configuration of the SLOM object, 
    * and developers are discouraged to use this method directly.</p>
    * 
    * @param kos A list with the KnowledgeObjects to configure
    */
    public void configureKnowledgeObjects(ArrayList<KnowledgeObject> kos) {
    
      // 1 retrieve all Axioms in ccm (see printOWLAxioms method)
        Set<OWLAxiom> all = owlOntCCM.getAxioms();
       
      // for each KnowledgeObject in kos, 
       for (KnowledgeObject ko : kos) {          
           for (OWLAxiom ax : all) {
               if (ax instanceof OWLAnnotationAssertionAxiom) { 
                   OWLAnnotationAssertionAxiom aax = (OWLAnnotationAssertionAxiom)ax;
                   
                   if (ko.getId()!= null && (ko.getId().equals("<"+aax.getSubject()+">")
                                             || ko.getId().equals(aax.getSubject())
                                             || aax.getSubject().equals("<"+ko.getId()+">"))
                                             )
                   {
                       ko.addProperty(aax.getProperty().toString(),aax.getValue().toString());                       
                   }
               }
           }                      
        } 
        
        // also, add microLPs
        
        
    }
    
    /**
    *  Search the OWLOntology for KO relationships and return them.
    * 
    * <p>This method queries the Axioms of the CCM OWLOntology and finds 
    * the relationships between KnowledgeObjects. For each of the 
    * relationships found, builds the <i>KnowledgeObjectRelationship</i> object and 
    * adds it into a <i>Set</i>, then returns this Set.</p>
    *
    * <p>This provides a high abstraction level mean to read the content of the CCM.</p>
    * 
    * @return a Set with the KnowledgeObjectRelationship found
    * @see org.intuitel.slom.KnowledgeObjectRelationship
    * @see org.intuitel.slom.KnowledgeObject
    */
    public Set<KnowledgeObjectRelationship> getKnowledgeObjectRelationships() {

        Set<KnowledgeObjectRelationship> KORelations = new HashSet();

        //Create factories that will produce the objects
        OWLReasonerFactory reasonerFactory = new StructuralReasonerFactory();
        OWLDataFactory fac = man.getOWLDataFactory();

        //Get a reasoner, to query the ontology
        OWLReasoner reasoner = reasonerFactory.createReasoner(owlOntCCM);

        //Get MicroLP relations. They contain the name of the MLP and the related CCs
        OWLClass myMicroLPclass = fac.getOWLClass(IRI.create("http://www.intuitel.eu/public/intui_PO.owl#MicroLearningPathway"));
        NodeSet<OWLNamedIndividual> individualsMicroLPNodeSet = reasoner.getInstances(myMicroLPclass, false);
        Set<OWLNamedIndividual> microLPRelations = individualsMicroLPNodeSet.getFlattened();

        Set<String> microLPTypes = new HashSet();
        //Get MicroLP types
        for (OWLNamedIndividual milpr : microLPRelations) {
            /* ********************************************************
              the milpr is the relation, I can create the KORelations
            ********************************************************** */
            Set<OWLClassExpression> typesOfIndividual = milpr.getTypes(owlOntCCM);
            String microLPName = typesOfIndividual.iterator().next().toString();
            KORelations.add(new KnowledgeObjectRelationship(milpr.toString(), microLPName));

        }
        configureKORelations(KORelations);
        return KORelations;
    }

    /**
    * Search MicroLearningPathways in this CCM and return a list of them.
    *
    * <p>This method queries the Axioms of the CCM OWLOntology and finds 
    * the MicroLearningPathways. For each MicroLearningPathway
    * found, builds the <i>MicroLP</i> Java representation
    * object and adds it into an <i>ArrayList</i>, then returns this ArrayList.</p>
    *
    * <p>This provides a high abstraction level mean to read the content of the CCM.</p>
    * 
    * @return an ArrayList with the MicroLP found in the CCM
    * @see org.intuitel.slom.MicroLP    
    */
    public ArrayList<MicroLP> getMicroLPs(){
        return getMicroLPs( getKnowledgeObjectRelationships() );
    }

    
    /**
    * Search MicroLearningPathways in this CCM filtered by KORels, and returns a list of them.
    *
    * <p>This method receives a <i>Set</i> of KnowledgeObjectRelationship and retrieves from
    * them the list of MicroLP defined at these KnowledgeObjectRelationships.
    * For each MicroLearningPathway identified, builds the <i>MicroLP</i> Java representation
    * object and adds it into an <i>ArrayList</i>, then returns this ArrayList.</p>
    *
    * <p>This provides a high abstraction level mean to read the content of the CCM.</p>
    * 
    * @return an ArrayList with the MicroLP found in the CCM, filtered by KORelations
    * @param KORelationships A set with the KORelationships used as filter
    * @see org.intuitel.slom.MicroLP    
    */
    public ArrayList<MicroLP> getMicroLPs(Set<KnowledgeObjectRelationship> KORelationships){
        ArrayList<MicroLP> alMLP = new ArrayList();
        MicroLP currentMLP = null;
        MicroLP testMLP = null;

        for (KnowledgeObjectRelationship kor : KORelationships) {
            testMLP = new MicroLP( kor.getMicroLP() );
            if ( alMLP.contains( testMLP ) ) {

                //obtener una referencia y añadir la relacion
                currentMLP = alMLP.get( alMLP.indexOf(testMLP) );
                currentMLP.addKnowledgeObjectRelationship(kor);
                currentMLP.addKnowledgeObject(new KnowledgeObject(kor.getHasPredecessorInMicroLp()));
                currentMLP.addKnowledgeObject(new KnowledgeObject(kor.getHasSuccessorInMicroLp()));
            } else {
                //crear el mlp, añadir la relación y meterlo al set
                testMLP.addKnowledgeObjectRelationship(kor);
                alMLP.add(testMLP);
                testMLP.addKnowledgeObject(new KnowledgeObject(kor.getHasPredecessorInMicroLp()));
                testMLP.addKnowledgeObject(new KnowledgeObject(kor.getHasSuccessorInMicroLp()));
            }

        }

        Iterator<MicroLP> it = alMLP.iterator();
        while(it.hasNext()) {
	    MicroLP mlp = it.next();
	    configureKnowledgeObjects(new ArrayList(mlp.getKnowledgeObjects()));
        }        

        return alMLP;
    }
  
    /**
    * Configures the MicroLP received as parameter.
    * 
    * <p>Adds to the <i>mlp</i> those KnowledgeObject from <i>KOs</i> that belong
    * to the MicroLP.</p>
    *
    * <p>This method is used for the initial configuration of the SLOM object, 
    * and developers are discouraged to use this method directly.</p>
    * 
    * @param KOs A list of KOs to include in the MicroLP
    * @param mlp The MicroLP to configure
    * @see org.intuitel.slom.MicroLP
    * @see org.intuitel.slom.KnowledgeObject
    */
    public void configureMicroLP(List<KnowledgeObject> KOs, MicroLP mlp) {

        Iterator<KnowledgeObject> it= KOs.iterator();
        while(it.hasNext()) {
            KnowledgeObject ko = it.next();
            
            if(ko.getMicroLPs().contains(mlp)){
                mlp.addKnowledgeObject(ko);
            }
        }
    }
}
