/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.slom;

import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Vector;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;

import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.rdf.model.SimpleSelector;
import com.hp.hpl.jena.rdf.model.InfModel;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.reasoner.ValidityReport;
import com.hp.hpl.jena.ontology.OntModelSpec;

import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.*;
import org.semanticweb.owlapi.io.*;
import org.semanticweb.owlapi.util.SimpleIRIMapper;

import org.semanticweb.owlapi.reasoner.OWLReasoner;
import org.semanticweb.owlapi.reasoner.OWLReasonerConfiguration;
import org.semanticweb.owlapi.reasoner.OWLReasonerFactory;
import org.semanticweb.owlapi.reasoner.SimpleConfiguration;
import org.semanticweb.owlapi.reasoner.structural.StructuralReasoner;
import org.semanticweb.owlapi.reasoner.structural.StructuralReasonerFactory;
import org.semanticweb.owlapi.model.OWLDataProperty; 
import org.semanticweb.owlapi.model.PrefixManager; 
import org.semanticweb.owlapi.util.DefaultPrefixManager; 

import org.semanticweb.owlapi.reasoner.NodeSet;
import org.semanticweb.owlapi.reasoner.ConsoleProgressMonitor;
import org.apache.commons.io.IOUtils;

import org.coode.owlapi.manchesterowlsyntax.ManchesterOWLSyntaxOntologyFormat;
import org.semanticweb.owlapi.io.OWLFunctionalSyntaxOntologyFormat;
import uk.ac.manchester.owl.owlapi.tutorialowled2011.OWLTutorialSyntaxOntologyFormat;
import org.semanticweb.owlapi.io.OWLXMLOntologyFormat;
import org.coode.owlapi.turtle.TurtleOntologyFormat;
import org.semanticweb.owlapi.vocab.PrefixOWLOntologyFormat;


/**
* Models the CognitiveMap part of the SLOM format.
*
* <p>The SLOM course description in a SLOM package is provided by two OWL files, 
* representing the CognitiveMap and the CognitiveContentMap. Where: <p>
*
* <ul>
* <li>The Cognitive Map refers to the concepts that are included in a certain 
* Knowledge Domain. Each of this concept is expressed as a ConceptContainer.</li>
* <li>The Cognitive Content Map refers to the actual course material that 
* can be used to teach/learn those concepts. Each of these part of course 
* material are expressed as KnowledgeObject.</li>
* </ul>
*
* <p>This class provides a Java representation of the <b>CognitiveMap</b> part, based on the use
* of the OWLOntology object of the OWLAPI library. This Java class 
* is internally used to create the SLOM object, but developers are discouraged to
* directly use this class.</p>
*
* <p>Further information of the SLOM format can be found at the 
* <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
* 
* @author  Luis de-la-Fuente-Valentín
* @since   1.0
*/
public class CognitiveMap
{
    private OWLOntology owlOntCM;
    private OWLOntologyManager man;

    /**
    * Returns the OWLOntology object that represents the CM.
    *
    * <p>The internal representation of the CognitiveMap is based on the OWLAPI library. 
    * This method provides access to the OWLOntology object.</p>
    * <p>The OWLOntology object can be used by developers to execute advanced operation
    * on the SLOM content, such as reasoning, complex queries, etc.</p>
    *
    * @return the OWLOntology object that represents the CM
    */  
    public OWLOntology getOWLOntology() {
        return owlOntCM;
    }

    /**
    * Configures the MacroLP received as parameter.
    * 
    * <p>Adds to the <i>mlp</i> those ConceptContainer from <i>CCs</i> that belong
    * to the MacroLP.</p>
    *
    * <p>This method is used for the initial configuration of the SLOM object, 
    * and developers are discouraged to use this method directly.</p>
    * 
    * @param CCs A list of CCs to include in the MacroLP
    * @param mlp The MacroLP to configure
    * @see org.intuitel.slom.MacroLP
    * @see org.intuitel.slom.ConceptContainer    
    */
    public void configureMacroLP(List<ConceptContainer> CCs, MacroLP mlp) {

        Iterator<ConceptContainer> it= CCs.iterator();
        while(it.hasNext()) {
            ConceptContainer cc = it.next();
            if(cc.getMacroLPs().contains(mlp)){
                mlp.addConceptContainer(cc);
            }
        }
    }

    /**
    * Search ConceptContainers in this CM, filter by MacroLP and return the results.
    *
    * <p>This method queries the Axioms of the CM OWLOntology and finds 
    * the ConceptContainers. Then, filters the results and select those CCs that 
    * part of any of the MacroLP received  as parameters. Finally, 
    * builds the <i>ConceptContainer</i> Java representation
    * objects and adds them into an <i>ArrayList</i>, then returns this ArrayList.</p>
    *
    * <p>This provides a high abstraction level mean to read the content of the CM.</p>
    * 
    * @param alMacroLPs the list of MacroLP used as filter
    * @return an ArrayList with the ConceptContainer found in the CM, filtered by MacroLP
    * @see org.intuitel.slom.ConceptContainer
    * @see org.intuitel.slom.MacroLP
    */
    public ArrayList<ConceptContainer> getConceptContainers(ArrayList<MacroLP> alMacroLPs){
        ArrayList<ConceptContainer> alCC = new ArrayList();
        List<String> ccsAsString = null;
        ConceptContainer newCC = null;
        ConceptContainer currentCC = null;

        for (MacroLP mlp : alMacroLPs) {
            ccsAsString = mlp.getConceptContainersAsString();

            for (String s : ccsAsString) {
                newCC = new ConceptContainer(s);
                if (alCC.contains(newCC) ){
                    currentCC = alCC.get( alCC.indexOf(newCC) );
                    currentCC.addMacroLP(mlp);
                } else {
                    newCC.addMacroLP(mlp);
                    alCC.add(newCC);
                }
            }
        }

        return alCC;
    }

    
    /**
    * Search MacroLearningPathways in this CM filtered by CCRels, and returns a list of them.
    *
    * <p>This method receives a <i>Set</i> of ConceptContainerRelationships and retrieves from
    * them the list of MacroLP that defined at these ConceptContainerRelationships.
    * For each MacroLearningPathway identified, builds the <i>MacroLP</i> Java representation
    * object and adds it into an <i>ArrayList</i>, then returns this ArrayList.</p>
    *
    * <p>This provides a high abstraction level mean to read the content of the CM.</p>
    * 
    * @param setCCRelationships a Set with the CCRelaionships used as filter
    * @return an ArrayList with the MacroLP found in the CM, filtered by CCRelations
    * @see org.intuitel.slom.MacroLP    
    */
    public ArrayList<MacroLP> getMacroLPs(Set<ConceptContainerRelationship> setCCRelationships){
        ArrayList<MacroLP> alMLP = new ArrayList();
        MacroLP currentMLP = null;
        MacroLP testMLP = null;

        for (ConceptContainerRelationship ccr : setCCRelationships) {
            testMLP = new MacroLP( ccr.getMacroLP() );
            if ( alMLP.contains( testMLP ) ) {

                //obtener una referencia y añadir la relacion
                currentMLP = alMLP.get( alMLP.indexOf(testMLP) );
                currentMLP.addConceptContainerRelationship(ccr);
            } else {
                //crear el mlp, añadir la relación y meterlo al set
                testMLP.addConceptContainerRelationship(ccr);
                alMLP.add(testMLP);
            }
        }

        return alMLP;
    }

    /**
    * Search MacroLearningPathways in this CM and return a list of them.
    *
    * <p>This method queries the Axioms of the CM OWLOntology and finds 
    * the MacroLearningPathways. For each MacroLearningPathway
    * found, builds the <i>MacroLP</i> Java representation
    * object and adds it into an <i>ArrayList</i>, then returns this ArrayList.</p>
    *
    * <p>This provides a high abstraction level mean to read the content of the CM.</p>
    * 
    * @return an ArrayList with the MacroLP found in the CM
    * @see org.intuitel.slom.MacroLP    
    */
    public ArrayList<MacroLP> getMacroLPs(){
        return getMacroLPs( getConceptContainerRelationships() );
    }

    /**
    * Search ConceptContainers in this CM and return a list of them.
    *
    * <p>This method queries the Axioms of the CM OWLOntology and finds 
    * the ConceptContainers. For each CC
    * found, builds the <i>ConceptContainer</i> Java representation
    * object and adds it into an <i>ArrayList</i>, then returns this ArrayList.</p>
    *
    * <p>This provides a high abstraction level mean to read the content of the CM.</p>
    * 
    * @return an ArrayList with the ConceptContainer found in the CM
    * @see org.intuitel.slom.ConceptContainer    
    */
    public ArrayList<ConceptContainer> getConceptContainers(){
        return getConceptContainers( getMacroLPs() );
    }

    /**
    * Prints in the standard output all the axioms in the CM.
    *
    * <p>Reads all the axioms at the CM and print them at the standard output. 
    * Useful for debuggin purposes</p>
    */  
    public void printOWLAxioms(){
        Set<OWLAxiom> all = owlOntCM.getAxioms();
        Iterator<OWLAxiom> it = all.iterator();
//        Vector<OWLAxiom> ccs = new Vector();

        OWLAxiom current = null;
        while (it.hasNext()) {
            current = it.next();
            System.out.println(current);
            System.out.println("============================");
//            if ( current.getAxiomType().toString().equals("ClassAssertion")) {
//                OWLClassAssertionAxiom a = (OWLClassAssertionAxiom)current;
//                if (a.getClassExpression().toString().equals("<"+SLOMUtil.INTUI_KO_IRI+">") ) {
//                    ccs.add(current);
//                    System
//                }
//            }
        }
    } 

    /**
    *  Search the OWLOntology for CC relationships and return them.
    * 
    * <p>This method queries the Axioms of the CM OWLOntology and finds 
    * the relationships between concept containers. For each of the 
    * relationships found, builds the <i>ConceptContainerRelationship</i> object and 
    * adds it into a <i>Set</i>, then returns this Set.</p>
    *
    * <p>This provides a high abstraction level mean to read the content of the CM.</p>
    * 
    * @return a Set with the ConceptContainerRelationship found
    * @see org.intuitel.slom.ConceptContainerRelationship
    * @see org.intuitel.slom.ConceptContainer    
    */
    public Set<ConceptContainerRelationship> getConceptContainerRelationships() {

        Set<ConceptContainerRelationship> setCCRelations = new HashSet();

        //Create factories that will produce the objects
        OWLReasonerFactory reasonerFactory = new StructuralReasonerFactory();
        OWLDataFactory fac = man.getOWLDataFactory();

        //Get a reasoner, to query the ontology
        OWLReasonerConfiguration config = new SimpleConfiguration();
        OWLReasoner reasoner = reasonerFactory.createReasoner(owlOntCM, config);

        //Get MacroLP relations. They contain the name of the MLP and the related CCs
        OWLClass myMLPclass = fac.getOWLClass(IRI.create("http://www.intuitel.eu/public/intui_PO.owl#MacroLearningPathway"));
        NodeSet<OWLNamedIndividual> individualsMLPNodeSet = reasoner.getInstances(myMLPclass, false);
        Set<OWLNamedIndividual> macroLPRelations = individualsMLPNodeSet.getFlattened();

        //Inspect each MLP and create relationship objects
        for (OWLNamedIndividual mlp : macroLPRelations) {

            /* ********************************************************
              the mlp is the relation, I can create the CCRelations
            *********************************************************** */

            Set<OWLClassExpression> typesOfIndividual = mlp.getTypes(owlOntCM);
            String macroLPName = typesOfIndividual.iterator().next().toString();

            Map<OWLObjectPropertyExpression,Set<OWLIndividual>> relationIndividuals = mlp.getObjectPropertyValues(owlOntCM);
            Set<OWLObjectPropertyExpression> keys = relationIndividuals.keySet();

            String thePrevCC = null;
            String theNextCC = null;
            for (OWLObjectPropertyExpression k: keys) {

                String theCC =  relationIndividuals.get(k).iterator().next().toString();
                boolean isHasNext = k.toString().equals("<http://www.intuitel.eu/public/intui_PO.owl#hasSuccessorInMacroLp>");
                if (isHasNext) {
                    theNextCC = theCC;
                }
                boolean isHasPrev = k.toString().equals("<http://www.intuitel.eu/public/intui_PO.owl#hasPredecessorInMacroLp>");
                if (isHasPrev) {
                    thePrevCC =  theCC;
                }
            }
            ConceptContainerRelationship theCCRelation = new ConceptContainerRelationship(mlp.toString(),
                                                                                          macroLPName,
                                                                                          thePrevCC,
                                                                                          theNextCC);
            setCCRelations.add(theCCRelation);
        }
        return setCCRelations;
    }

    private void createManager(Iterator<SimpleIRIMapper> iriMaps){
       man = OWLManager.createOWLOntologyManager(); 
       while (iriMaps.hasNext()){
            man.addIRIMapper(iriMaps.next());
       }
    }

    /**
    * Returns the CM IRI, as read from the owl file
    *
    * @return The CM IRI, as read from the owl file
    */
    public IRI getCMIRI() {
        if (owlOntCM == null){
            System.err.println("The CM object was not properly initialized");
            System.exit(1);
        }
        if (owlOntCM.getOntologyID() == null){
            System.err.println("The CM object does not contain OntologyID");
            System.exit(1);
        }
        IRI owlid = owlOntCM.getOntologyID().getDefaultDocumentIRI();
        return owlid;
    }

    /**
    * Builds the CM object from owl content.
    * 
    * <p>The CM object is build by reading the information from the OWL content
    * passed as parameter.</p>
    *
    * @param cmIn the CM as OWL content
    * @param iriMaps a container with a IRI mapping to use while loading the owl content 
    */
    public CognitiveMap(InputStream cmIn, Iterator<SimpleIRIMapper> iriMaps) {
       createManager(iriMaps);
       try {
           owlOntCM = man.loadOntologyFromOntologyDocument(cmIn);// throws OWLOntologyCreationException
       } catch (OWLOntologyCreationException e) {
           System.err.println("Problem while reading the ontology with OWL API");
           e.printStackTrace();
           System.exit(1);
       }
    }

    /**
    * Determines if the received file contains a CM description.
    *     
    * <p>The decision is based on the SLOM format definition, further information can be found at the 
    * <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
    * 
    *
    * @param cmIn the CM content to validate
    * @param iriMaps a container with a IRI mapping to use while loading the owl content 
    * @return true if it is valid, false otherwise.¡        
    */
    public static boolean isValidCM(InputStream cmIn, Iterator<SimpleIRIMapper> iriMaps) {

       boolean isLoadable = false;
       boolean containsCC = false;
       boolean isConsistent = false;

       OWLOntologyManager iriManager = OWLManager.createOWLOntologyManager(); 
       while (iriMaps.hasNext()){
            iriManager.addIRIMapper(iriMaps.next());
       }

       OWLOntology testOwlOnt = null;
       try {
           //if throws an exception, then it is not valid
           testOwlOnt = iriManager.loadOntologyFromOntologyDocument(cmIn);
           isLoadable = true;

           ///////////////////////////////////////////////
           //basic check (quite fool). If contains CCs
           ///////////////////////////////////////////////
           //Create factories that will produce the objects
           OWLReasonerFactory reasonerFactory = new StructuralReasonerFactory();
           OWLDataFactory fac = iriManager.getOWLDataFactory();

           //Get a reasoner, to query the ontology
           OWLReasoner reasoner = reasonerFactory.createReasoner(testOwlOnt);

           //Get CCs
           OWLClass myClass = fac.getOWLClass(IRI.create("http://www.intuitel.eu/public/intui_PO.owl#ConceptContainer"));
           NodeSet<OWLNamedIndividual> ccs = reasoner.getInstances(myClass, false);
           Set<OWLNamedIndividual> ccsFlat = ccs.getFlattened();

           containsCC = (ccsFlat.size() > 0);

           ///////////////////////////////////////////////
           //advanced. Check for consistency with a reasoner
           //////////////////////////////////////////////
           reasoner.precomputeInferences();
           isConsistent = reasoner.isConsistent();

       } catch (OWLOntologyCreationException e) {
           System.err.println("Problem while reading the ontology with OWL API");
           e.printStackTrace();
           isLoadable = false;
       }

       return (isLoadable && containsCC && isConsistent); 
    }


    /** 
    * Writes the content of the CM as OWL/RDF.
    *
    * <p>This method uses a RDFXMLOntologyFormat to serialized the ontology information
    * and write it in the <i>os</i> OutputStream.</p> 
    *
    * @param os The OutputStream where the information will be written
    */      
    public void write(OutputStream os) {
       try {
            man.saveOntology(owlOntCM, new RDFXMLOntologyFormat(), os);
       }catch (OWLOntologyStorageException e) {
           System.err.println("Problem while writing the ontology with OWL API");
           e.printStackTrace();
           System.exit(1);
       }
    }
}
