/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.slom;

import java.util.*;

/**
*
* Java representation of INTUITEL Concept Container.
*
* <p>The SLOM course description establishes that the course content is organized 
* as Concept Containers, which describe the course topics at a higher 
* level of abstraction.<p>
*
* <p>Concept Containers refers to an idea that should be learn
* in a certain Knowlowedge Domain. They are organized with Macro 
* Learning Pathways and contains Knowledge Objects as low level 
* information items.</p>
*
* <p>This class provides a Java representation of the <b>Concept Container</b>.</p>
*
* <p>Further information of the SLOM format can be found at the 
* <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
* 
* @author  Luis de-la-Fuente-Valentín
* @since   1.0
*/
public class ConceptContainer {

    private String id;
    private String title;
    private String description;
    private ArrayList<MicroLP> microLPs;//Contained MicroLPs
    private ArrayList<MacroLP> macroLPs;//Where this concept container is present

    /**
    *
    * Creates an empty ConceptContainer.
    *
    * <p>Creates a ConceptContainer object, with the id given as parameter and with empty lists 
    * of MacroLPs and MicroLPs</p>
    *
    * <p>Therefore, ConceptContainer configuration is required to complete the object setup</p>
    *
    * @param id The id of the ConceptContainer
    */
    public ConceptContainer(String id) {
        this.id=id;
        microLPs = new ArrayList();
        macroLPs = new ArrayList();
    }

    
    /**
    * Returns the id of this ConceptContainer.
    * 
    * 
    * @return the id of this ConceptContainer.
    */
    public String getId() {
        return id;
    }

    /**
    * TWO ConceptContainers are equal if they have the same id.
    *
    * @param o the ConceptContainer to compare
    * @return true if this ConceptContainer has the same id as the ConceptContainer received in <i>o</i>.
    */
    public boolean equals (Object o) {
        ConceptContainer CCO;
        try {
            CCO = (ConceptContainer)o;
        } catch (ClassCastException e) {
            return false;
        }

        return (this.id.equals(CCO.getId()));
    }

    
    /**
    * Provides a textual description of this ConceptContainer.
    *
    * <p>The textual description of this ConceptContainer contains its id, title 
    * and description. Also, lists the textual description of all MacroLP and MicroLP 
    * in the ConceptContainer</p>
    *
    * @return The String with the textual description
    */
    public String toString() {
        String s = "";
        s = s + "id: " + id + "\n";
        s = s + "title: " + title + "\n";
        s = s + "description: " + description + "\n";

        s = s + "MacroLPs:\n";
        if (macroLPs != null) {
            Iterator<MacroLP> it = macroLPs.iterator();
            while (it.hasNext()) {
                s = s + "\n" + it.next().getName();
            }
        }
        s = s + "\nMicroLPs:\n (sin implementar)";

        return s;
    }

    /**
    * Sets the id of this ConceptContainer.
    * 
    * @param id The id to set.
    */
    public void setId(String id) {
        this.id = id;
    }

    /**
    * Gets the title of this ConceptContainer.
    * 
    * @return The title of this ConceptContainer.
    */    
    public String getTitle() {
        return title;
    }

    /**
    * Sets the title of this ConceptContainer.
    * 
    * @param title The title to set.
    */        
    public void setTitle(String title) {
        this.title = title;
    }

    /**
    * Gets the description of this ConceptContainer.
    * 
    * @return The description of this ConceptContainer.
    */        
    public String getDescription() {
        return description;
    }

    /**
    * Sets the description of this ConceptContainer.
    * 
    * @param description The description to set.
    */
    public void setDescription(String description) {
        this.description = description;
    }

    
    /**
    * Gets the list of MicroLPs in this ConceptContainer.
    * 
    * @return A list with the MicroLP in this ConceptContainer
    */         
    public ArrayList<MicroLP> getMicroLPs() {
        return microLPs;
    }

    /**
    * Sets the list of MicroLPs in this ConceptContainer.
    * 
    * @param microLPs A list with the MicroLP to include in this ConceptContainer
    */    
    public void setMicroLPs(ArrayList<MicroLP> microLPs) {
        this.microLPs = microLPs;
    }

    
    /**
    * Gets the list of MacroLPs in this ConceptContainer.
    * 
    * @return A list with the MacroLP in this ConceptContainer
    */     
    public ArrayList<MacroLP> getMacroLPs() {
        return macroLPs;
    }

    /**
    * Sets the list of MacroLPs in this ConceptContainer.
    * 
    * @param macroLPs A list with the MacroLP to include in this ConceptContainer
    */        
    public void setMacroLPs(ArrayList<MacroLP> macroLPs) {
        this.macroLPs = macroLPs;
    }

    /**
    * Adds one MacroLP to this ConceptContainer.
    * 
    * @param mlp The MacroLP to add
    */        
    public void addMacroLP(MacroLP mlp) {
        if ( !macroLPs.contains(mlp) ) {
            macroLPs.add(mlp);
        }
    }

}

