/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.slom;

/**
*
* Java representation of INTUITEL Concept Container Relationship.
*
* <p>The SLOM course description establishes that Concept Containers are sequenced
* with Macro Learning Pathways, and those MacroLP contains sequencing rules called
* conceptContainerRelationships.Each ConceptContainerRelationship referst to two 
* ConceptContainer, the predecessor and the successor.</p>
*
* <p>This class provides a Java representation of the <b>Concept Container Relationship</b>.</p>
*
* <p>Further information of the SLOM format can be found at the 
* <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
* 
* @author  Luis de-la-Fuente-Valentín
* @since   1.0
*/
public class ConceptContainerRelationship {

    private String name;
    private String macroLP;
    private String hasPredecessorInMacroLp;
    private String hasSuccessorInMacroLp;


    /**
    * Builds the ConceptContainerRelationship.
    *
    *
    * <p>Builds the ConceptContainerRelationship from its name, the macroLP 
    * that contains the relationship, the predecessor CC and the successor CC.</p>
    *
    * @param name the name of the Relationship to create
    * @param macroLP the macroLP that will contain the relationship
    * @param hasPredecessorInMacroLp The ConceptContainer with the predecessor role in the relationship
    * @param hasSuccessorInMacroLp The ConceptContainer with the sucessor role in the relationship
    */
    public ConceptContainerRelationship(String name,
                                        String macroLP,
                                        String hasPredecessorInMacroLp,
                                        String hasSuccessorInMacroLp) {
        this.name = name;
        this.macroLP = macroLP;
        this.hasPredecessorInMacroLp = hasPredecessorInMacroLp;
        this.hasSuccessorInMacroLp = hasSuccessorInMacroLp;
    }

    /**
    * Provides a textual representation of the Relationship.
    * 
    * <p>The textual representation contains the name, the MacroLP, the predecessor CC and the sucessor CC</p>
    *
    * @return the textual representation
    */
    public String toString() {
        String s= "";
        s = s + "name: " + name + "\n";
        s = s + "MacroLP: " + macroLP + "\n";
        s = s + "hasPredecessorInMacroLp: " +hasPredecessorInMacroLp + "\n";
        s = s + "hasSuccessorInMacroLp: " +hasSuccessorInMacroLp + "\n";
        return s;
    }

    /** 
    * Returns the name of this ConceptContainerRelationship.
    * 
    * @return The name of the ConceptContainerRelationship
    */
    public String getName() {
        return name;
    }

    /**
    * Establishes the name of the ConceptContainerRelationship.
    *
    * @param name the name to set.
    */    
    public void setName(String name) {
        this.name = name;
    }

    /** 
    * Returns the name of the macroLP of this ConceptContainerRelationship.
    * 
    * @return The name of the macroLP of the ConceptContainerRelationship
    */    
    public String getMacroLP() {
        return macroLP;
    }

    /**
    * Establishes the name of the MacroLP of the ConceptContainerRelationship.
    *
    * @param macroLP the name to set.
    */        
    public void setMacroLP(String macroLP) {
        this.macroLP = macroLP;
    }

    
    /** 
    * Returns the name of the ConceptContainer that is the predecessor in the ConceptContainerRelationship.
    * 
    * @return The name of the ConceptContainer that is the predecessor in the ConceptContainerRelationship.
    */    
    public String getHasPredecessorInMacroLp() {
        return hasPredecessorInMacroLp;
    }

    /**
    * Establishes the name of the ConceptContainer that is the predecessor in the ConceptContainerRelationship.
    *
    * @param hasPredecessorInMacroLp the name of the CC to set.
    */
    public void setHasPredecessorInMacroLp(String hasPredecessorInMacroLp) {
        this.hasPredecessorInMacroLp = hasPredecessorInMacroLp;
    }

    /** 
    * Returns the name of the ConceptContainer that is the sucessor in the ConceptContainerRelationship.
    * 
    * @return The name of the ConceptContainer that is the sucessor in the ConceptContainerRelationship.
    */    
    public String getHasSuccessorInMacroLp() {
        return hasSuccessorInMacroLp;
    }

    /**
    * Establishes the name of the ConceptContainer that is the sucessor in the ConceptContainerRelationship.
    *
    * @param hasSuccessorInMacroLp the name of the CC to set.
    */
    public void setHasSuccessorInMacroLp(String hasSuccessorInMacroLp) {
        this.hasSuccessorInMacroLp = hasSuccessorInMacroLp;
    }

}
