/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
 package org.intuitel.merger;

import java.io.File;
import java.io.OutputStream;
import java.io.IOException;
import java.util.Set;
import java.util.Iterator;
import org.intuitel.slom.SLOM;

/**
 * Abstract class that defines the interface used by the software using the Merger.
 *
 *
 * <p>The Serializer class defines two methods: marshall and unmarshall. The former
 * is used to generate XML content representing a given SLOM object. The latter is
 * used to read XML content and create a SLOM object with the information read.</p>
 *
 * <p>Subclasses of Serializer implement those methods and accomodate
 * the proccess to the particular needs of the XML format in use. 
 * As different serialization formats require the use of different subclasses 
 * of Serializer, a SerializerFactory should be used to obtain the required Serializer.</p>
 *
 * <ul>
 * <li>Developers who want to use the Merger a 
 * software library have to create Serializer objects</li>
 * <li>Developers who want to use the extend the Merger functionality by implementing
 * transformations to other formats, should extend the Serializer class. 
 * software library have to create Serializer objects</li>
 * </ul>
 
 *
 * @see SerializerFactory
 */
public abstract class Serializer
{
    /** 
    * Reads content from a file and returns a SLOM object with the information read.
    *
    * <p>The unmarshall method reads content from a file and returns a SLOM object with
    * the information read.</p>
    *
    * <p>The method accepts a Set of File objects, but in most cases the
    * Set will only contain a single File. This is to support those cases in which the
    * information require several files to be complete.</p>
    * 
    * <p>Currently supported input formats are SCORM, SLOM, IMSLD and 
    * Semantic Media Wiki</p>
    * 
    * @param inputCourse A Set of files containing the course information in the desired format
    * @return A SLOM object containing the information read from the input
    */
    public abstract SLOM unmarshall (Set<File> inputCourse) throws BadFileException, IOException;


    /** 
    * Writes the information stored in a SLOM object in a OutputStream.
    *
    * <p>The marshall method serializes a SLOM object using the serialization
    * supported by the subclass of Serializer that is being used.
    * </p>
    *
    * <p>The method accepts the SLOM object to be serialized, and a Iterator of 
    * OutputStream objects. In the case of IMSLD and SCORM, for instance, only one 
    * OutputStream is required since all the information fits in the imsmanifest.xml file.
    * In the case of SLOM, two OutputStream are required, to store the CM and the CCM 
    * respectively.</p>
    *   
    * <p>The method returns nothing, since the generated information is written in
    * the OutputStream objects passed as parameter.</p>
    *
    * <p>Currently supported input formats are SCORM, SLOM and IMSLD. Marshalling is not
    * supported for Semantic Media Wiki</p>
    * 
    * @param theSlom A SLOM object containing the information to be serialized
    * @param it A Iterator of OutputStream objects, that will be used to write the serialization
    */
    public abstract void marshall (SLOM theSlom, Iterator<OutputStream> it) throws InvalidParameterException, IOException ;


}
