/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.merger;

/**
 * Creates Serializer objects.
 *
 * <p>The SerializerFactory class defines a single (overloaded) method, which results
 * in the creation of serializer objects.</p>
 * 
 * <p>Serializer is an abstract class that simply defines the interface for the
 * actual serializer implementations. The SerializerFactory creates objects 
 * of the Serializer subclasses and return them shaped as Serializer</p>
 *
 * <p>That means, developers should use this class in order to obtain Serializer objects. 
 * As a good practice, developers who extend the merger functionality by supporting 
 * more formats are encouraged to extend the SerializerFactory clas.</p>
 *
 * @see Serializer
 */
public class SerializerFactory
{
  /** Creates a SLOM Serializer object.
    * 
    * <p>As default configuration, the getSerializer method with no parameters
    *    returns a SLOMSerializer object. To obtain serializers from and to 
    *    other formats, use <i>Serializer getSerializer(String inputFormat)</i></p>
    *
    * @return A Serializer object that marshalls and unmarshalls SLOM content
    * @see #getSerializer(String inputFormat)
    */
    public static Serializer getSerializer() throws InvalidParameterException  {
        Serializer s;
        // slom is the default value
        s = new SLOMSerializer();
        return s;
    }

    /** Creates a Serializer object.
    * 
    * <p>Creates and returns a Serializer object, whose working format can be specified as 
    * a parameter. Allowed formats are: </p>
    * <ul>
    * <li><i>imsld</i>: IMS Learning Design course packages </li>
    * <li><i>scorm</i>: ADL Scorm course packages</li>
    * <li><i>slom</i>: SLOM course packages</li>
    * <li><i>smw</i>: Course defined at Semantic Media Wiki</li>    
    * </ul>
    *
    * @param format The translation format that this Serializer will translate
    * @return A Serializer object that marshalls and unmarshalls the type of content given by format
    * @see #getSerializer(String inputFormat)
    */
    public static Serializer getSerializer(String format) throws InvalidParameterException {
        Serializer s; 

        if ("imsld".equals(format)) {
                s = new IMSLDSerializer();
        }
        else if ("scorm".equals(format)) {
                s = new SCORMXSLTSerializer();
        }
        else if ("slom".equals(format)) {
                s = new SLOMSerializer();
        }
        else if ("smw".equals(format)) {
                s = new SMWSerializer();
        } else {
                throw new InvalidParameterException(format);
        }
        return s;
    } 
}

