/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.slom;

/**
*
* Java representation of INTUITEL Knowledge Object Relationship.
*
* <p>The SLOM course description establishes that Knowledge Objects are sequenced
* with Micro Learning Pathways, and those MicroLP contains sequencing rules called
* KnowledgeObjectRelationships. Each KnowledgeObjectRelationship refers to two 
* KnowledgeObjects: the predecessor and the successor.</p>
*
* <p>This class provides a Java representation of the <b>Knowledge Object Relationship</b>.</p>
*
* <p>Further information of the SLOM format can be found at the 
* <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
* 
* @author  Luis de-la-Fuente-Valentín
* @since   1.0
*/
public class KnowledgeObjectRelationship {

    private String name;
    private String microLP;
    private String hasPredecessorInMicroLp;
    private String hasSuccessorInMicroLp;


    /**
    * Builds the KnowledgeObjectRelationship.
    *
    * <p>Builds the KnowledgeObjectRelationship from its name</p>
    *
    * @param name the name of the Relationship to create
    */    
    public KnowledgeObjectRelationship(String name) {
        setName(name);
    }

    /**
    * Builds the KnowledgeObjectRelationship.
    *
    * <p>Builds the KnowledgeObjectRelationship from its name and microLP</p>
    *
    * @param name the name of the Relationship to create
    * @param microLP the name of the microLP in the Relationship to create
    */   
    public KnowledgeObjectRelationship(String name, String microLP) {
        setName(name);
        setMicroLP(microLP);
    }

    /**
    * Provides a textual representation of the Relationship.
    * 
    * <p>The textual representation contains the name, the MicroLP, the predecessor KO and the sucessor KO</p>
    *
    * @return the textual representation
    */
    public String toString() {
        String s= "";
        s = s + "name: " + name + "\n";
        s = s + "MicroLP: " + microLP + "\n";
        s = s + "hasPredecessorInMicroLp: " +hasPredecessorInMicroLp + "\n";
        s = s + "hasSuccessorInMicroLp: " +hasSuccessorInMicroLp + "\n";
        return s;
    }

    /** 
    * Returns the name of this KnowledgeObjectRelationship.
    * 
    * @return The name of the ConceptContainerRelationship
    */    
    public String getName() {
        return name;
    }

    /**
    * Establishes the name of the KnowledgeObjectRelationship.
    *
    * @param name the name to set.
    */          
    public void setName(String name) {
        this.name = name;
    }

    /** 
    * Returns the name of the KnowledgeObject that is the predecessor in the KnowledgeObjectRelationship.
    * 
    * @return The name of the KnowledgeObject that is the predecessor in the KnowledgeObjectRelationship.
    */       
    public String getHasPredecessorInMicroLp() {
        return hasPredecessorInMicroLp;
    }

    /** 
    * Returns the name of the MicroLP in this KnowledgeObjectRelationship.
    * 
    * @return The name of the MicroLP in this KnowledgeObjectRelationship
    */  
    public String getMicroLP() {
        return microLP;
    }

    /**
    * Establishes the name of the MicroLP of the KnowledgeObjectRelationship.
    *
    * @param microLP the name to set.
    */      
    public void setMicroLP(String microLP) {
        this.microLP = microLP;
    }

    /**
    * Establishes the name of the KnowledgeObject that is the predecessor in the KnowledgeObjectRelationship.
    *
    * @param hasPredecessorInMicroLp the name of the KO to set.
    */    
    public void setHasPredecessorInMicroLp(String hasPredecessorInMicroLp) {
        this.hasPredecessorInMicroLp = hasPredecessorInMicroLp;
    }

    /** 
    * Returns the name of the KnowledgeObject that is the sucessor in the KnowledgeObjectRelationship.
    * 
    * @return The name of the KnowledgeObject that is the sucessor in the KnowledgeObjectRelationship.
    */       
    public String getHasSuccessorInMicroLp() {
        return hasSuccessorInMicroLp;
    }

    /**
    * Establishes the name of the KnowledgeObject that is the sucessor in the KnowledgeObjectRelationship.
    *
    * @param hasSuccessorInMicroLp the name of the KO to set.
    */        
    public void setHasSuccessorInMicroLp(String hasSuccessorInMicroLp) {
        this.hasSuccessorInMicroLp = hasSuccessorInMicroLp;
    }

}
