/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.slom;

import java.util.*;




/**
*
* Java representation of INTUITEL Macro Learning Pathways.
*
* <p>The SLOM course description establishes that Concept Containers are sequenced
* with Macro Learning Pathways.<p>
*
* <p>Each Macro Learning Pathway contains a collection of Concept Containers and also
* the relationships among them.</p>
*
* <p>This class provides a Java representation of the <b>Macro Learning Pathway</b>.</p>
*
* <p>Further information of the SLOM format can be found at the 
* <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
* 
* @author  Luis de-la-Fuente-Valentín
* @since   1.0
*/
public class MacroLP {

    private String id;
    private String name;
    private String description;
    private String predefinedMacroLP;
    private List<ConceptContainer> conceptContainers;//Concept containers for this macroLP
    private List<ConceptContainerRelationship> conceptContainerRelationships;//Concept containers relationships
    private String pathToDiagramFile;

    /**
    *
    * Creates an empty MacroLP.
    *
    * <p>Creates a MacroLP object, with the name given as parameter and with empty lists 
    * of ConceptContainers and ConceptContainerRelationships</p>
    *
    * <p>Therefore, MacroLP configuration is required to complete the object setup</p>
    *
    * @param name The name of the MacroLP
    */
    public MacroLP(String name){
        this.name = name;
        this.conceptContainers = new ArrayList();
        this.conceptContainerRelationships = new ArrayList();
    }

    /**
    * Provides a list of Strings representing the ConceptContainers in this MacroLP.
    *
    * <p>Builds and returns a list of the textual representation of the ConceptContainers
    * in this MacroLP.</p>
    *
    * @return A list of Strings with the textual description of the ConceptContainers
    */
    public List<String> getConceptContainersAsString() {

        ArrayList<String> output = new ArrayList();
        String prevCC = null;
        String nextCC = null;
        Iterator<ConceptContainerRelationship> it = conceptContainerRelationships.iterator();
        while (it.hasNext()) {
            ConceptContainerRelationship myccr = it.next();
            prevCC = myccr.getHasPredecessorInMacroLp();
            nextCC = myccr.getHasSuccessorInMacroLp();

            if (!output.contains(prevCC)){
                output.add(prevCC);
            }

            if (!output.contains(nextCC)){
                output.add(nextCC);
            }
        }

        return output;
    }

    /**
    * Provides a textual description of this MacroLP.
    *
    * <p>The textual description of this MacroLP contains its name
    * and a list of the textual description of all ConceptContainerRelationship
    * in the MacroLP</p>
    *
    * @return The String with the textual description
    */
    public String toString() {
        String s = "\n";
        s = s + "name: " + name + "\n";

        s = s + "Concept Container Relationships:\n";
        if (conceptContainerRelationships != null){
            Iterator<ConceptContainerRelationship> it = conceptContainerRelationships.iterator();
            while (it.hasNext()) {
                s = s + "\n" + it.next().toString();
            }
        }
        return s;
    }


    /**
    * Two MacroLP are equal if the have the same name.
    *
    *
    * @return True if this object has the same name as the MacroLP received as parameter
    */
    public boolean equals (Object o) {
        MacroLP mlpO;
        try {
            mlpO = (MacroLP)o;
        } catch (ClassCastException e) {
            return false;
        }

        return (this.name.equals(mlpO.getName()));
    }

    
    /**
    * Returns the id of this MacroLP.
    *
    * @return The id of this macroLP
    */
    public String getId() {
        return id;
    }

    /**
    * Establishes the identifier of this MacroLP.
    *
    * @param id The String to be set as identifier.
    */
    public void setId(String id) {
        this.id = id;
    }

    /**
    * Returns the name of this MacroLP.
    *
    * @return The name of this macroLP
    */
    public String getName() {
        return name;
    }

    /**
    * Establishes the name of this MacroLP.
    *
    * @param name The String to be set as name.
    */
    public void setName(String name) {
        this.name = name;
    }

    /**
    * Returns the description of this MacroLP.
    *
    * @return The description of this macroLP
    */    
    public String getDescription() {
        return description;
    }

    /**
    * Establishes the description of this MacroLP.
    *
    * @param description The String to be set as description.
    */
    public void setDescription(String description) {
        this.description = description;
    }


    /**
    * Returns a list of the ConceptContainers in this this MacroLP.
    *
    * @return a list of the ConceptContainers in this this MacroLP.
    */
    public List<ConceptContainer> getConceptContainers() {
        return conceptContainers;
    }

    /**
    * Establishes the ConceptContainers of this MacroLP.
    *
    * @param conceptContainers a list witht the ConceptContainers to set.
    */    
    public void setConceptContainers(List<ConceptContainer> conceptContainers) {
        this.conceptContainers = conceptContainers;
    }

    /**
    * Adds a concept container to this MacroLP.
    *
    * @param cc The ConceptContainer to add
    */
    public void addConceptContainer(ConceptContainer cc) {
        this.conceptContainers.add(cc);
    }

    /**
    * Get a list of the ConceptContainerRelationship in this MacroLP.
    *
    * @return a list of the ConceptContainerRelationship in this MacroLP.
    **/
    public List<ConceptContainerRelationship> getConceptContainerRelationships() {
        return conceptContainerRelationships;
    }

    /**
    * Establishes the ConceptContainerRelationships of this MacroLP.
    *
    * @param conceptContainerRelationships a list witht the ConceptContainerRelationship to set.
    */ 
    public void setConceptContainerRelationships(List<ConceptContainerRelationship> conceptContainerRelationships) {
        this.conceptContainerRelationships = conceptContainerRelationships;
    }

    /**
    * Adds a concept container relationshiop to this MacroLP.
    *
    * @param ccr The ConceptContainerRelationship to add
    */
    public void addConceptContainerRelationship(ConceptContainerRelationship ccr) {

        this.conceptContainerRelationships.add(ccr);
    }

    public String getPathToDiagramFile() {
        return pathToDiagramFile;
    }

    public void setPathToDiagramFile(String pathToDiagramFile) {
        this.pathToDiagramFile = pathToDiagramFile;
    }

    
    public String getPredefinedMacroLP() {
        return predefinedMacroLP;
    }

    public void setPredefinedMacroLP(String predefinedMacroLP) {
        this.predefinedMacroLP = predefinedMacroLP;
    }
}
