/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.slom;

import java.util.*;

/**
*
* Java representation of INTUITEL Micro Learning Pathways.
*
* <p>The SLOM course description establishes that Knowledge Objects are sequenced
* with Micro Learning Pathways.<p>
*
* <p>Each Micro Learning Pathway contains a collection of Knowledge Objects and also
* the relationships among them.</p>
*
* <p>This class provides a Java representation of the <b>Micro Learning Pathway</b>.</p>
*
* <p>Further information of the SLOM format can be found at the 
* <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
* 
* @author  Luis de-la-Fuente-Valentín
* @since   1.0
*/
public class MicroLP {

    private String id;
    private String name;
    private String description;
    private String predefinedMicroLP;
    private Set<KnowledgeObject> knowledgeObjects = new HashSet<KnowledgeObject>();
    private ArrayList<KnowledgeObjectRelationship> knowledgeObjectRelationships;//Knowledgeobjects relationships
    private String pathToDiagramFile;

    /**
    *
    * Creates an empty MicroLP.
    *
    * <p>Creates a MicroLP object, with the name given as parameter and with empty lists 
    * of KnowledgeObject and KnowledgeObjectRelationship</p>
    *
    * <p>Therefore, MicroLP configuration is required to complete the object setup</p>
    *
    * @param name The name of the MicroLP
    */
    public MicroLP(String name) {
        this(name,null);
    }
 
    /**
    *
    * Creates an empty MicroLP.
    *
    * <p>Creates a MicroLP object, with the name given as parameter and with empty lists 
    * of KnowledgeObject and KnowledgeObjectRelationship</p>
    *
    * <p>Therefore, MicroLP configuration is required to complete the object setup</p>
    *
    * @param name The name of the MicroLP    
    * @param id The id of the MicroLP
    */
    public MicroLP(String name, String id) {
        setId(id);
        setName(name);
        knowledgeObjects = new HashSet<KnowledgeObject>();
        knowledgeObjectRelationships = new ArrayList();    }

    /**
    * Provides a textual description of this MicroLP.
    *
    * <p>The textual description of this MicroLP contains its name
    * and a list of the textual description of all KnowledgeObjectRelationships and KnowledgeObjects
    * in the MicroLP</p>
    *
    * @return The String with the textual description
    */
    public String toString() {
        String s = "\n";
        s = s + "name: " + name + "\n";
        s = s + "Knowledge Object Relationships:\n";
        if (knowledgeObjectRelationships != null){
            Iterator<KnowledgeObjectRelationship> it = knowledgeObjectRelationships.iterator();
            while (it.hasNext()) {
                s = s + "\n" + it.next().toString();
            }
        }
        s = s + "Knowledge Objects:\n";
        if (knowledgeObjects != null){
            Iterator<KnowledgeObject> it = knowledgeObjects.iterator();
            while (it.hasNext()) {
                s = s + "\n" + it.next().toString();
            }
        }
        return s;
    }

    /**
    * Two MicroLP are equal if the have the same name.
    *
    * @return True if this object has the same name as the MicroLP received as parameter
    */
    public boolean equals (Object o) {
        MicroLP mlpO;
        try {
            mlpO = (MicroLP)o;
        } catch (ClassCastException e) {
            return false;
        }

         if (this.name == null || mlpO == null || mlpO.getName() == null)
             return false;
            
        return (this.name.equals(mlpO.getName()));
    }

    /**
    * Returns the id of this MicroLP.
    *
    * @return The id of this microLP
    */
    public String getId() {
        return id;
    }

    /**
    * Establishes the identifier of this MicroLP.
    *
    * @param id The String to be set as identifier.
    */    
    public void setId(String id) {
        this.id = id;
    }

    /**
    * Returns the name of this MicroLP.
    *
    * @return The name of this microLP
    */
    public String getName() {
        return name;
    }
    
    /**
    * Establishes the name of this MicroLP.
    *
    * @param name The String to be set as name.
    */  
    public void setName(String name) {
        this.name = name;
    }

    /**
    * Returns the description of this MicroLP.
    *
    * @return The description of this microLP
    */    
    public String getDescription() {
        return description;
    }

    /**
    * Establishes the description of this MacroLP.
    *
    * @param description The String to be set as description.
    */
    public void setDescription(String description) {
        this.description = description;
    }


    /**
    * Returns a set with the KnowledgeObjects in this this MicroLP.
    *
    * @return a set with the KnowledgeObjects in this this MicroLP.
    */    
    public Set<KnowledgeObject> getKnowledgeObjects() {
        return knowledgeObjects;
    }

    /**
    * Establishes the KnowledgeObjects of this MicroLP.
    *
    * @param knowledgeObjects a list witht the KnowledgeObject to set.
    */  
    public void setKnowledgeObjects(Set<KnowledgeObject> knowledgeObjects) {
        this.knowledgeObjects = knowledgeObjects;
    }

    /**
    * Adds a knowledge object to this MicroLP.
    *
    * @param knowledgeObject The KnowledgeObject to add
    */
    public void addKnowledgeObject(KnowledgeObject knowledgeObject) {
        if (knowledgeObject != null) {
            this.knowledgeObjects.add(knowledgeObject);
        }
    }

    /**
    * Returns a set with the KnowledgeObjectRelationships in this this MicroLP.
    *
    * @return a set with the KnowledgeObjectRelationships in this this MicroLP.
    */    
    public List<KnowledgeObjectRelationship> getKnowledgeObjectRelationships() {
        return knowledgeObjectRelationships;
    }

    /**
    * Establishes the KnowledgeObjectRelationships of this MicroLP.
    *
    * @param knowledgeObjectRelationships a list witht the KnowledgeObjectRelationship to set.
    */     
    public void setKnowledgeObjectRelationships(ArrayList<KnowledgeObjectRelationship> knowledgeObjectRelationships) {
        this.knowledgeObjectRelationships = knowledgeObjectRelationships;
    }

    /**
    * Adds a knowledge object relationship to this MicroLP.
    *
    * @param kor The KnowledgeObjectRelationship to add
    */
    public void addKnowledgeObjectRelationship(KnowledgeObjectRelationship kor) {
        this.knowledgeObjectRelationships.add(kor);
    }
    
    /**
    * Provides a list of Strings representing the KnowledgeObjects in this MicroLP.
    *
    * <p>Builds and returns a list of the textual representation of the KnowledgeObjects
    * in this MicroLP.</p>
    *
    * @return A list of Strings with the textual description of the KnowledgeObjects
    */
    public List<String> getKnowledgeObjectsAsString() {

        ArrayList<String> output = new ArrayList();
        String prevKO = null;
        String nextKO = null;
        Iterator<KnowledgeObjectRelationship> it = knowledgeObjectRelationships.iterator();
        while (it.hasNext()) {
            KnowledgeObjectRelationship mykor = it.next();
            prevKO = mykor.getHasPredecessorInMicroLp();
            nextKO = mykor.getHasSuccessorInMicroLp();

            if (!output.contains(prevKO)){
                output.add(prevKO);
            }

            if (!output.contains(nextKO)){
                output.add(nextKO);
            }
        }

        return output;
    }
    
    /**
    *
    * Cleans the list of knowledgeObjects in this MicroLP by removing duplicates.
    *
    */
    public void removeDuplicates(){
        Set<KnowledgeObject> uniqueKOs = new HashSet();
        for (KnowledgeObject ko: knowledgeObjects) {
            if (!uniqueKOs.contains(ko) ){
                 uniqueKOs.add(ko);
            }        
        }
        knowledgeObjects = uniqueKOs;
    }
    

    public String getPathToDiagramFile() {
        return pathToDiagramFile;
    }

    public void setPathToDiagramFile(String pathToDiagramFile) {
        this.pathToDiagramFile = pathToDiagramFile;
    }

    public String getPredefinedMicroLP() {
        return predefinedMicroLP;
    }

    public void setPredefinedMicroLP(String predefinedMicroLP) {
        this.predefinedMicroLP = predefinedMicroLP;
    }    
}
