/*
 * Creator: INTUITEL Consortium, www.intuitel.eu
 * Contributor: Luis de-la-Fuente-Valentín, Universidad Internacional de La Rioja, UNIR, www.unir.net
 * Version: 1
 * Date: 2014-09-29
 * Copyright: Copyright (C) Universidad Internacional de La Rioja, UNIR
 * License: This software is provided as freeware. 

 Freeware products are provided free of charge. Freeware software may be used as many 
 times as you like, for as long as you like. You may copy and distribute copies of 
 this program provided that you keep this copyright notice and disclaimer of warranty
 intact, and recipients must be bound by the terms therein. You must not charge money
 or fees for the software product to anyone except to cover distribution costs. You 
 must acknowledge and cite the authorship of this software by UNIR. The copyright 
 holder reserves the right to reclassify this software as a non-freeware product at 
 any time (i.e. shareware, licensed royalty software, or public domain). Doing so will
 not modify the license agreement of previously distributed executables.
 */
package org.intuitel.slom;

import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.net.URL;
import java.util.ArrayList;

import org.intuitel.merger.BadFileException;
import org.apache.commons.io.IOUtils;
import org.semanticweb.owlapi.model.*;
import org.semanticweb.owlapi.util.SimpleIRIMapper;
import org.semanticweb.owlapi.apibinding.OWLManager;

/**
* This class is the main representation of a SLOM metadata.
*
* <p>The SLOM object is a Java representation of the SLOM format defined by the INTUITEL 
* project. A SLOM object can be created from its OWL/RDF representation, or it can be 
* the result of a transformation from a supported format. In any case, the serialization
* is executed by a <i>Serizalizer</i> object.
* </p>
*
* <p>As created from a file, the SLOM object already contains the course information, 
* which can be read but not edited. Acording to the SLOM specification, the course metadata
* is organized as: </p>
*
* <ul>
* <li>ConceptContainer objects</li>
* <li>KnowledgeObject objects</li>
* <li>MacroLP objects</li>
* <li>MicroLP objects</li>
* </ul>
*
* <p>Internally, the SLOM object works with CM and CCM objects which can be accessed 
* via <i>getCCM</i> and <i>getCM</i> methods. These objects work with the OWLAPI library 
* and could be used for editing purposes, despite developers are encouraged not to do so.</p>
*
* <p>Further information of the SLOM format can be found at the 
* <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
* 
* @see org.intuitel.merger.Serializer
* @author  Luis de-la-Fuente-Valentín
* @since   1.0
*/
public class SLOM
{
    private CognitiveMap cm;
    private CognitiveContentMap ccm;


    private ArrayList<ConceptContainer> conceptContainers;
    private ArrayList<MacroLP> macroLPs;
    private ArrayList<KnowledgeObject> knowledgeObjects;
    private ArrayList<MicroLP> microLPs;

    /* Used to be able to load the imported ontologies*/
    private LinkedList<SimpleIRIMapper> sim;

    /** 
    * This method returns the list of the MacroLPs.
    *
    * <p>Returns the MacroLP objects contained in this SLOM course</p>
    *
    * @return An ArrayList with all the list of the macroLPs 
    *         contained in this SLOM metadata
    */
    public ArrayList<MacroLP> getMacroLPs() {
        return macroLPs;
    }
    
    /** 
    * Returns the list of the ConceptContainers.
    *
    * <p>Returns the ConceptContainer objects in this SLOM course</p>
    *
    * @return An ArrayList with all the list of the ConceptContainer 
    *         contained in this SLOM metadata
    */    
    public ArrayList<ConceptContainer> getConceptContainers() {
        return conceptContainers;
    }
    
    /** 
    * Returns the list of the KnowledgeObjects.
    *
    * <p>Returns the KnowledgeObject objects in this SLOM course</p>
    *
    * @return An ArrayList with all the list of the KnowledgeObject 
    *         contained in this SLOM metadata
    */      
    public ArrayList<KnowledgeObject> getKnowledgeObjects() {
        return knowledgeObjects;
    }

    /** 
    * Returns the list of the MicroLPs.
    *
    * <p>Returns the MicroLP objects in this SLOM course</p>
    *
    * @return An ArrayList with all the list of the microLPs 
    *         contained in this SLOM metadata
    */  
    public List<MicroLP> getMicroLPs() {
        return microLPs;
    }

    /** 
    * Returns the list of the MicroLPs in a ConceptContainer.
    *
    * <p>Returns the MicroLP objects, from this SLOM course, that are used in a given ConceptContainer</p>
    *
    * @param conceptContainer The conceptContainer to search in
    * @return An ArrayList with all the list of the ConceptContainer
    */  
    public List<MicroLP> getMicroLPs(ConceptContainer conceptContainer) {
                
        //get microLPs
        ArrayList<MicroLP> filteredMicroLPs= new ArrayList();                        
        
        for (MicroLP mlp: microLPs) {         
            Set<KnowledgeObject> myKos = new HashSet();
            Set<KnowledgeObject> kos = mlp.getKnowledgeObjects();
            for (KnowledgeObject ko:kos) {
                //for each KO, check if it is contained in one of the conceptContainers
                for (KnowledgeObject configuredKo:knowledgeObjects) {
                 
                    if (configuredKo.getId()!=null && ko.getId()!=null) {
                        if (configuredKo.getId().equals("<"+ko.getId()+">")) {
                            myKos.add(configuredKo);
                        }
                    }
                }                
            }            
            
            //en este punto, myKos contiene la lista de KOs que están en este microLP
            for (KnowledgeObject myKo:myKos) {                
                String cc = myKo.getProperty("<http://www.intuitel.eu/public/intui_PO.owl#isContainedByConceptContainer>");                
                String a = conceptContainer.getId();                
                String b = "<"+cc+">";
                                
                if (a!=null && a.equals(b)) {                    
                    if (!filteredMicroLPs.contains(mlp)){
                        filteredMicroLPs.add(mlp);                                            
                        break;
                    }
                }                     
            }
        }
        
        
        
//         for (KnowledgeObject ko:knowledgeObjects) {
//             ko.printKO();
//         }
        return filteredMicroLPs;
        //return filteredMicroLPs;
        
    }

    /**
    * Creates the SLOM object from a SLOM package.
    *
    * @param SLOM_file The file that contains the SLOM package
    */
    public SLOM(File SLOM_file) throws BadFileException, IOException {
        sim = new LinkedList();
        this.readFile(SLOM_file);
        configureSLOM();
    }

    /**
    * Creates the SLOM object from its respective CM and CCM.
    *
    * @param cm The content of the CM file
    * @param ccm The content of the CCM file
    */
    public SLOM(InputStream cm, InputStream ccm) throws BadFileException, IOException {
        //call parse functions
        sim = new LinkedList();
        readCMandCCM(cm,ccm);
        configureSLOM();
    }

    private void configureSLOM() {
        /* this line just initilizes the macroLPs, which means that they
           have most of the attributes, but still do not have the list
           of conceptContainers */
        macroLPs = cm.getMacroLPs();
        conceptContainers = cm.getConceptContainers(macroLPs);

        /* now we can configure the list of concept containers 
           as part of the MacroLP */
        for (MacroLP mlp : macroLPs) {
            cm.configureMacroLP(conceptContainers, mlp);
        }

        knowledgeObjects = ccm.getKnowledgeObjects();
        ccm.configureKnowledgeObjects(knowledgeObjects);          
        
        microLPs = ccm.getMicroLPs();
        for (MicroLP mlp : microLPs) {         
            mlp.removeDuplicates();
        }

//        ccm.printOWLAxioms();
    }

    /**
    * Returns the OWLOntology object that represents the CM.
    *
    * <p>The internal representation of the CognitiveMap is based on the OWLAPI library. 
    * This method provides access to the OWLOntology object.</p>
    * <p>The OWLOntology object can be used by developers to execute advanced operation
    * on the SLOM content, such as reasoning, complex queries, etc.</p>
    *
    * @return the OWLOntology object that represents the CM
    * @see #getCognitiveMap()
    */    
    public OWLOntology getCM() {
        return cm.getOWLOntology();
    }

    /**
    * Returns the CognitiveMap object of this SLOM.
    *
    * @return the the CognitiveMap object of this SLOM
    * @see #getCM
    */    
    public CognitiveMap getCognitiveMap() {
        return cm;
    }
    
    /**
    * Returns the OWLOntology object that represents the CCM.
    *
    * <p>The internal representation of the ContentCognitiveMap is based on the OWLAPI library. 
    * This method provides access to the OWLOntology object.</p>
    * <p>The OWLOntology object can be used by developers to execute advanced operation
    * on the SLOM content, such as reasoning, complex queries, etc.</p>
    *
    * @return the OWLOntology object that represents the CCM
    * @see #getCognitiveContentMap()
    */ 
    public OWLOntology getCCM() {
        return ccm.getOWLOntology();
    }

    /**
    * Returns the ContentCognitiveMap object of this SLOM.
    *
    * @return the the ContentCognitiveMap object of this SLOM
    * @see #getCCM
    */ 
    public CognitiveContentMap getCognitiveContentMap() {
        return ccm;
    }

    private static String inputStreamToFile(InputStream in, String path) throws FileNotFoundException, IOException
    {
       String tDir = System.getProperty("java.io.tmpdir");
       String tmpPath = tDir + "/" + path;

       FileOutputStream fos = new FileOutputStream(tmpPath);
       IOUtils.copy(in, fos);

       return tmpPath;
    }

    private void readCMandCCM(InputStream cm, InputStream ccm)  throws FileNotFoundException, IOException
    {
        //call parse functions
        initSim();
        IRI cmIRI = parseCMStream(cm,sim.iterator());

        String cmAbsPath = inputStreamToFile(cm, SLOMUtil.INTUI_CM_FILE);
        File fcm = new File(cmAbsPath);

        sim.add( new SimpleIRIMapperHelper(cmIRI, IRI.create(fcm) ) );

        parseCCMStream(ccm, sim.iterator());
        flushSim();
    }
    
    /**
    * Determines if the received file contains a SLOM package.
    *     
    * <p>The decision is based on the SLOM format definition, further information can be found at the 
    * <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
    * 
    *
    * @param cm the CM content of the SLOM to validate
    * @param ccm the CCM content of the SLOM to validate     
    * @return a String with the ERROR code resulting from the validation
    *
    * @see org.intuitel.slom.SLOMError    
    */
    public static String isValidSLOM(InputStream cm, InputStream ccm) throws IOException 
    {

        /* mind that this method is static, so we need to replicate here the
           creation of the manager and the iriMaps, because the methods that 
           build these objects are non-static (and cannot be changed)
        */

        //create iriMaps, useful for CM and CCM parsing
        InputStream poIn = SLOM.class.getResourceAsStream(SLOMUtil.INTUI_PO_FILE);
        String poAbsPath = SLOM.inputStreamToFile(poIn, SLOMUtil.INTUI_PO_FILE);
        File fpo = new File(poAbsPath);
        LinkedList<SimpleIRIMapper> iriMaps = new LinkedList();
        IRI a = SLOMUtil.INTUI_PO;
        IRI b = IRI.create(fpo);
        SimpleIRIMapperHelper simp = new SimpleIRIMapperHelper(a, b);
        iriMaps.add(simp);

        // CM contains ConceptContainers
        // CM check for consistency by using a reasoner
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        IOUtils.copy(cm, baos);

        InputStream is1 = new ByteArrayInputStream(baos.toByteArray()); 
        InputStream is2 = new ByteArrayInputStream(baos.toByteArray());

        if( !CognitiveMap.isValidCM( is1, iriMaps.iterator() ) )
        {
            SLOMError se = SLOMError.CM_NOT_VALID;
            System.err.println(se);
            return se.getCode();
        }

        OWLOntologyManager myOOM = OWLManager.createOWLOntologyManager(); 
        Iterator<SimpleIRIMapper> it = iriMaps.iterator();
        while (it.hasNext()){
             myOOM.addIRIMapper(it.next());
        }
        OWLOntology myTmpCM = null;
        try {
            myTmpCM = myOOM.loadOntologyFromOntologyDocument(is2);// throws OWLOntologyCreationException

        } catch (OWLOntologyCreationException e) {
            System.err.println("This should not happen, I have been already done this");
            e.printStackTrace();
        }

        IRI tmpCMIRI = myTmpCM.getOntologyID().getDefaultDocumentIRI();
        String cmAbsPath = SLOM.inputStreamToFile(cm, SLOMUtil.INTUI_CM_FILE);
        File fcm = new File(cmAbsPath);
        
        iriMaps.add( new SimpleIRIMapperHelper(tmpCMIRI, IRI.create(fcm) ) );

        // CCM contains Knowledge Objects
        // CCM check for consistency by using a reasoner
        if( !CognitiveContentMap.isValidCCM( ccm, iriMaps.iterator() ) )
        {
            SLOMError se = SLOMError.CCM_NOT_VALID;
            System.err.println(se);
            return se.getCode();
        }

        return SLOMError.VALID.getCode();
    }

    /**
    * Determines if the received file contains a SLOM package.
    *     
    * <p>The decision is based on the SLOM format definition, further information can be found at the 
    * <a href="http:www.intuitel.de/resources">INTUITEL project page</a>, at deliverable D4.1</p>
    * 
    *
    * @param SLOM_file The SLOM  package to validate.    
    * @return a String with the ERROR code resulting from the validation
    *
    * @see org.intuitel.slom.SLOMError    
    */
    public static String isValidSLOM(File SLOM_file) throws IOException 
    {
        //checks if a file  contains a valid SLOM 

        //that is, checks if the zip contains CM and CCM files, 
        //and check if they are what they are supposed to be

        // the file is needed. There is no need for a method that 
        // checks validity on "this" object, because if it has 
        // been imported as an object, it is supposed to be valid
        ZipFile zfile = null;
        ZipEntry cmEntry = null;
        ZipEntry ccmEntry = null;;
        InputStream cmIn, ccmIn;

        // the received file is a ZIP file
        // contains CM and CCM in the expected formats
        //A SLOM File is a ZIP file with a CM and CCM files
        try {
            zfile = new ZipFile(SLOM_file);
        } catch (ZipException e) {
            SLOMError se = SLOMError.NO_ZIP_FILE;
            System.err.println(se);
            return se.getCode();
        }

        // There is a CM file
        cmEntry = zfile.getEntry(SLOMUtil.INTUI_CM_ZIPENTRY);
        if (cmEntry == null)
        {
            SLOMError se = SLOMError.NO_CM_FOUND;
            System.err.println(se);
            return se.getCode();
        }

        // There is a CCM file
        ccmEntry = zfile.getEntry(SLOMUtil.INTUI_CCM_ZIPENTRY);
        if (ccmEntry == null)
        {
            SLOMError se = SLOMError.NO_CCM_FOUND;
            System.err.println(se);
            return se.getCode();
        }

        cmIn = zfile.getInputStream(cmEntry);
        ccmIn = zfile.getInputStream(ccmEntry);

        return isValidSLOM(cmIn,ccmIn);
    }

    /**
      * puts the contents of the CM and CCM in the corresponding object fields
      **/
    private void readFile(File SLOM_file) throws BadFileException, IOException 
    {
        ZipFile zfile = null;
        ZipEntry cmEntry = null;
        ZipEntry ccmEntry = null;;
        InputStream cmIn, ccmIn;

        //receives a SLOM file and parses it to fill the object fields.
        if (isValidSLOM(SLOM_file) != SLOMError.VALID.getCode())
        {
            throw new BadFileException("The file does not contain a valid SLOM package");
        }

        //A SLOM File is a ZIP file with a CM and CCM files
        try {
            zfile = new ZipFile(SLOM_file);
        } catch (ZipException e) {
            System.err.println("This should not happend, because I have validated the file");
        }
         
        //open the zip and look for CM and CCM files
        cmEntry = zfile.getEntry(SLOMUtil.INTUI_CM_ZIPENTRY);
        ccmEntry = zfile.getEntry(SLOMUtil.INTUI_CCM_ZIPENTRY);

        //get their Strings and validate them
        //the file was valid SLOM, so no need to check 
        //if is null
        cmIn = zfile.getInputStream(cmEntry);
        ccmIn = zfile.getInputStream(ccmEntry);

        //call parse functions
        readCMandCCM(cmIn,ccmIn);
    }

    /**
      * Reads a zipentry, inside a zipfile, and returns its
      * content as a String
      */
    private String getStringFromEntry(ZipFile zfile, ZipEntry entry) 
    {
        String output = null;
        try {
            InputStream in = zfile.getInputStream(entry);
            BufferedReader reader = new BufferedReader(new InputStreamReader(in));
            StringBuilder out = new StringBuilder();
            String line;
            while ((line = reader.readLine()) != null) {
                out.append(line);
            }
            output = out.toString();   //Prints the string content read from input stream
            reader.close();
        } catch (IOException e) {
            System.err.println("Problem while unzipping ZIP file");
        }
        return output;
    }

    /**
      * creates the CognitiveMap, as read from the InputStream, and returns its IRI
      * the iriMap is used to load imported ontologies from local files, in case 
      * they cannot be loaded from the Internet
      */
    private IRI parseCMStream(InputStream cmIn,Iterator<SimpleIRIMapper> iriMaps )
    {
        //receives a CM InputStream and parses it to fill the cm fields.
        cm = new CognitiveMap(cmIn, iriMaps);
        if (cm == null) {
            System.err.println("The CM was not properly read, resulting in null cm object");
            System.exit(1);
        }
        return cm.getCMIRI();
    }

    /**
      * creates the CognitiveMap object and returns its IRI
      */
    private IRI parseCCMStream(InputStream ccmIn, Iterator<SimpleIRIMapper> iriMaps)
    {
        //receives a CCM string and parses it to fill the ccm field.
        ccm = new CognitiveContentMap(ccmIn, iriMaps);
        return ccm.getCCMIRI();
    }


    /** 
    * Writes the content of the CM as OWL/RDF.
    *
    * <p>This method is a wrapper of the <i>write</i> method provided by 
    * CognitiveMap objects.</p> 
    *
    * @param os The OutputStream where the information will be written
    * @see CognitiveMap
    */
    public void writeCM(OutputStream os) 
    {
        if (cm != null) {
            cm.write(os);
        }
    }

    /** 
    * Writes the content of the CCM as OWL/RDF.
    *
    * <p>This method is a wrapper of the <i>write</i> method provided by 
    * CognitiveContentMap objects.</p> 
    *
    * @param os The OutputStream where the information will be written
    * @see CognitiveContentMap
    */
    public void writeCCM(OutputStream os) 
    {
        if (ccm != null) {
            ccm.write(os);
        }
    }

    private void initSim() throws FileNotFoundException, IOException
    {
        InputStream poIn = SLOM.class.getResourceAsStream(SLOMUtil.INTUI_PO_FILE);
        String poAbsPath = SLOM.inputStreamToFile(poIn, SLOMUtil.INTUI_PO_FILE);
        File fpo = new File(poAbsPath);
        sim.add( new SimpleIRIMapperHelper(SLOMUtil.INTUI_PO,  IRI.create(fpo)) );
    }

    private void flushSim() 
    {
        SimpleIRIMapperHelper simh = null;
        String fileIRI = null;
        File f = null;
        Iterator<SimpleIRIMapper> it = sim.iterator();

        while(it.hasNext())
        {
            simh = (SimpleIRIMapperHelper)it.next();
            fileIRI = simh.getDocumentIRI().toString();

            if (fileIRI.startsWith("file:")) {
                fileIRI = fileIRI.substring(5);
            }

            f = new File(fileIRI);
            f.delete();
        }
    }

    private static class SimpleIRIMapperHelper extends SimpleIRIMapper {

        private IRI myDocumentIRI;
        private IRI myOntologyIRI;
    
        public SimpleIRIMapperHelper(IRI ontologyIRI, IRI documentIRI) {
            super(ontologyIRI, documentIRI);
            myDocumentIRI = documentIRI;
            myOntologyIRI = ontologyIRI;
        }

        public IRI getDocumentIRI() {
            return myDocumentIRI;
        }
        public IRI getOntologyIRI() {
            return myOntologyIRI;
        }
    }
}


